@@ -22,6 +22,7 @@
 #############################################################################*/
 VOID init_null();
 VOID testnull();
+int kleiner_w();
 int kleiner();
 int hat();
 int n_anwert();
@@ -29,27 +30,35 @@ int n_anspiel();
 int n_abwert();
 int n_abwerfen();
 int minmax();
+int minmaxfb();
 int drunter();
 int drunterdrue();
 VOID m_nsp();
 VOID m_nns();
 VOID null_stich();
+VOID null_sort();
+int null_dicht();
+VOID revolutiondist();
 VOID start_ramsch();
 VOID init_ramsch();
 int zweibuben();
 int bubeanspielen();
 int sicher();
 VOID moeglklein();
+VOID nimm_bube();
 VOID moegldrunter();
 int ggdurchmarsch();
 VOID m_bvr();
 VOID m_bmr();
 VOID m_bhr();
 VOID m_bramsch();
+int unsich_fb();
 int comp_sramsch();
 VOID ramsch_stich();
 VOID ramsch_result();
 int testgrandhand();
+int left();
+int right();
 int iscomp();
 VOID swap();
 VOID setrnd();
@@ -71,6 +80,9 @@ VOID do_entsch();
 VOID do_reizen();
 VOID drueck();
 VOID truempfe();
+int tr_voll();
+int sage_kontra();
+int sage_re();
 int testgrand();
 VOID calc_inhand();
 VOID calc_drueck();
@@ -79,12 +91,14 @@ VOID save_skat();
 int check_bockevents();
 VOID update_list();
 VOID do_grandhand();
+VOID set_prot();
 VOID do_handspiel();
 VOID do_handok();
 VOID do_ansagen();
 VOID karobube();
 int karobubespielen();
 VOID do_angesagt();
+VOID spielphase();
 int higher();
 VOID calc_result();
 VOID get_next();
@@ -97,6 +111,7 @@ VOID read_list();
 VOID fill_st();
 int maxnimm();
 VOID next_stich();
+VOID finishgame();
 VOID do_next();
 VOID calc_poss();
 VOID c_high();
@@ -111,13 +126,14 @@ int niedrighoch();
 int ueberdoerfer();
 int trumpfausspielen();
 int hochausspielen();
-VOID schenken();
+VOID schenke();
 int zehnblank();
 int fabwerfen();
 VOID abwerfen();
 int buttern();
 int hatas();
 int schnippeln();
+VOID nichtspitze();
 VOID m_bvsp();
 VOID m_bmsp();
 VOID m_bhsp();
@@ -164,6 +180,14 @@ VOID di_ansage();
 VOID di_kontra();
 VOID di_rekontra();
 VOID di_konre();
+VOID di_ktrnext();
+VOID di_dicht();
+VOID di_weiter();
+VOID di_klopfen();
+VOID di_schenken();
+VOID di_geschenkt();
+VOID di_wiederschenken();
+VOID di_nichtschenken();
 VOID di_schieben();
 int di_verdoppelt();
 VOID di_buben();
@@ -184,11 +208,11 @@ VOID di_bockevents();
 VOID di_verzoegerung();
 VOID di_menubutton();
 VOID di_wieder();
-int left();
 VOID change_gc();
 VOID change_gcbg();
 VOID change_gcxor();
 int istrue();
+VOID v_gtextnc();
 VOID v_gtextc();
 VOID v_gtext();
 VOID clr_text();
@@ -225,6 +249,9 @@ VOID movecard();
 VOID homecard();
 VOID givecard();
 VOID initscr();
+VOID nullendscr();
+VOID revolutionsort();
+VOID revolutionscr();
 VOID clr_desk();
 VOID draw_box();
 VOID put_box();
@@ -236,6 +263,7 @@ VOID inv_fbox();
 int card_at();
 int hndl_reizen();
 int hndl_druecken();
+int hndl_tauschen();
 int hndl_spielen();
 int hndl_nimmstich();
 VOID hndl_button();
