@@ -21,6 +21,18 @@
 #include "oldskat.h"
 #endif
 
+int left(s)
+int s;
+{
+  return (s+1)%3;
+}
+
+int right(s)
+int s;
+{
+  return (s+2)%3;
+}
+
 int iscomp(s)
 int s;
 {
@@ -273,7 +285,7 @@ VOID sort(sn)
 int sn;
 {
   int i,j,f=sn*10;
-  int hatfb[4],fbsum,firstf;
+  int hatfb[4],fbsum,firstf,sptz;
 
   sortw[0]=0;
   sortw[1]=1;
@@ -319,10 +331,17 @@ int sn;
       }
     }
   }
+  if (sn==spieler && spitzeang && !sort2[sn]) {
+    sptz=(trumpf==4?BUBE:SIEBEN|trumpf<<3);
+  }
+  else sptz=-2;
   for (i=f;i<f+9;i++) {
     for (j=i+1;j<f+10;j++) {
-      if (lower(cards[i],cards[j],sort2[sn])^sort1[sn])
-        swap(&cards[i],&cards[j]);
+      if (((cards[j]==sptz ||
+	    lower(cards[i],cards[j],sort2[sn])) &&
+	   cards[i]!=sptz)^sort1[sn]) {
+	swap(&cards[i],&cards[j]);
+      }
     }
   }
   sortw[0]=0;
@@ -397,7 +416,10 @@ int s;
        (bb+t[tr]==6))) maxrw[s]=18;
   stg=strateg[numsp==0?s:numsp==1?s-1:0];
   if (stg<0 && rnd(3)<-stg && maxrw[s] &&
-      maxrw[s]!=35) maxrw[s]=17;
+      maxrw[s]!=nullw[0] && maxrw[s]!=nullw[1] && maxrw[s]!=nullw[2] &&
+      maxrw[s]!=nullw[3] && maxrw[s]!=nullw[4]) {
+    maxrw[s]=17;
+  }
   if (stg>0 && rnd(3)<stg && !maxrw[s] &&
       (b[3] || b[2] || b[1])) maxrw[s]=17;
 }
@@ -407,11 +429,13 @@ VOID do_geben()
   int sn,i;
 
   sort2[0]=sort2[1]=sort2[2]=0;
-  schnang=schwang=ouveang=0;
+  prot2.verdopp[0]=prot2.verdopp[1]=prot2.verdopp[2]=0;
+  schnang=schwang=ouveang=spitzeang=revolang=0;
+  ndichtw=0;
   for (sn=0;sn<numsp;sn++) calc_desk(sn);
   if (!wieder) {
     if (ramschspiele) {
-      if (trumpf==4) geber=(geber+2)%3;
+      if (trumpf==4) geber=right(geber);
       else ramschspiele--;
     }
     else if (bockspiele) {
@@ -421,15 +445,15 @@ VOID do_geben()
       }
     }
     bockspiele+=3*bockinc;
-    geber=(geber+1)%3;
+    geber=left(geber);
   }
   else if (!vorhandwn) {
-    geber=(geber+wieder+1)%3;
+    geber=left(geber+wieder);
   }
   bockinc=0;
   trumpf=-1;
-  hoerer=(geber+1)%3;
-  sager=(geber+2)%3;
+  hoerer=ausspl=left(geber);
+  sager=right(geber);
   mischen();
   setcurs();
   givecard(hoerer,0);
@@ -448,6 +472,7 @@ VOID do_geben()
     sagte18[i]=0;
   }
   kontrastufe=0;
+  schenkstufe=0;
   saho=1;
   reizp=0;
   clear_info();
@@ -477,6 +502,7 @@ int s,w;
   inv_box(s,0);
   stdwait();
   inv_box(s,0);
+  sagte18[s]=1;
 }
 
 VOID do_passen(s)
@@ -497,6 +523,7 @@ int s;
   stdwait();
   inv_box(s,0);
   b_text(s," ");
+  sagte18[s]=1;
 }
 
 VOID do_msagen(sn,w)
@@ -639,6 +666,37 @@ VOID truempfe()
   }
 }
 
+int tr_voll(sn,f)
+int sn,f;
+{
+  int i,c,t,a,z;
+
+  if (trumpf==-1 || trumpf==4) return f;
+  t=a=z=0;
+  for (i=0;i<10;i++) {
+    c=cards[10*sn+i];
+    if ((c&7)==BUBE || c>>3==trumpf) t++;
+    else if ((c&7)==AS) a++;
+    else if ((c&7)==ZEHN) z++;
+  }
+  if (f) {
+    return t>7 || (t>6 && a+z);
+  }
+  return t>5 || (t>4 && a+z) || (t>3 && a>2);
+}
+
+int sage_kontra(sn)
+int sn;
+{
+  return tr_voll(sn,0);
+}
+
+int sage_re(sn)
+int sn;
+{
+  return tr_voll(sn,1);
+}
+
 int testgrand(bb,b,vh)
 int bb,b[4],vh;
 {
@@ -685,12 +743,18 @@ int sn;
 VOID calc_drueck()
 #undef calc_drueck
 {
-  int i,j,c,f,bb,n;
+  int i,j,c,f,bb,n,sp;
   int b[4],t[4],p[4],o[4];
 
-  if (maxrw[spieler]==35) {
+  if (maxrw[spieler]==nullw[0] ||
+      maxrw[spieler]==nullw[1] ||
+      maxrw[spieler]==nullw[2] ||
+      maxrw[spieler]==nullw[3] ||
+      maxrw[spieler]==nullw[4]) {
     trumpf=-1;
-    handsp=1;
+    if (maxrw[spieler]!=nullw[0] && maxrw[spieler]!=nullw[2]) handsp=1;
+    if (maxrw[spieler]>=nullw[3]) ouveang=1;
+    if (maxrw[spieler]==nullw[4]) revolang=1;
     gedr=2;
     return;
   }
@@ -744,6 +808,16 @@ VOID calc_drueck()
       testgrand(bb,b,spieler==ausspl)) {
     trumpf=4;
   }
+  if (spitzezaehlt &&
+      ((trumpf<4 && inhand[trumpf][SIEBEN] &&
+	(t[trumpf]+bb>=7 || (t[trumpf]+bb==6 && bb>=2))) ||
+       (trumpf==4 && b[0] &&
+	bb==3 && (spieler==ausspl || b[3]+b[2]==2)))) {
+    sp=trumpf==4?BUBE:SIEBEN|trumpf<<3;
+    if (cards[30]!=sp && cards[31]!=sp) {
+      spitzeang=1;
+    }
+  }
   gespcd[cards[30]]=1;
   gespcd[cards[31]]=1;
 }
@@ -781,7 +855,7 @@ int check_bockevents()
     if (!spgew && stsum==60 && trumpf>=0 && trumpf<=4) e++;
   }
   if (bockevents&BOCK_BEI_GRANDHAND) {
-    if (spgew && trumpf==4 && prot1.ehsso>0) e++;
+    if (spgew && trumpf==4 && handsp) e++;
   }
   m=0;
   if (bockevents&BOCK_BEI_KONTRA) {
@@ -840,27 +914,46 @@ int sn;
   do_handspiel();
 }
 
+VOID set_prot()
+{
+  prot2.stichgem=stich-1;
+  prot2.spieler=spieler;
+  prot2.trumpf=trumpf;
+  prot2.gereizt=reizp<0?0:reizw[reizp];
+  prot2.gewonn=spgew;
+  prot2.augen=stsum;
+  prot2.handsp=handsp;
+  prot2.ehsso=
+    revolang?handsp?1:0:
+    ouveang?trumpf==-1 && handsp?5:4:
+    schwang?3:schnang?2:handsp?1:0;
+  prot2.predef=predef;
+  if (trumpf!=5) prot2.sramsch=0;
+  prot2.savseed=savseed;
+  prot2.gamenr=gamenr-1;
+  prot2.rotateby=rotateby;
+  prot2.spitze=spitzeang;
+  prot2.revolution=revolang;
+  prot2.schenken=schenkstufe;
+}
+
 VOID do_handspiel()
 {
   int i;
 
+  prot2.anspiel[0]=ausspl;
+  prot2.gemacht[0]=-1;
   if (reizp<0 && !ramschspiele) {
     if (playramsch>0) {
       init_ramsch();
       return;
     }
-    clr_desk();
+    clr_desk(0);
     phase=GEBEN;
     stich=1;
     fill_st();
-    prot2.stichgem=0;
-    prot2.trumpf=4;
-    prot2.predef=predef;
-    prot2.savseed=savseed;
-    prot2.gamenr=gamenr-1;
-    prot2.rotateby=rotateby;
-    prot2.anspiel[0]=(geber+1)%3;
-    prot2.gemacht[0]=-1;
+    trumpf=4;
+    set_prot();
     save_skat(1);
     prot1=prot2;
     spwert=0;
@@ -877,7 +970,6 @@ VOID do_handspiel()
   stsum=0;
   vmh=0;
   gedr=0;
-  ausspl=(geber+1)%3;
   for (i=0;i<10;i++) spcards[i]=cards[spieler*10+i];
   save_skat(0);
   spcards[10]=cards[30];
@@ -901,7 +993,7 @@ VOID do_handok()
   }
   if (!iscomp(spieler) && !handsp) {
     draw_skat();
-    put_fbox(TX_DRUECKEN);
+    put_fbox(spieler,TX_DRUECKEN);
     phase=DRUECKEN;
   }
   else do_ansagen();
@@ -909,7 +1001,26 @@ VOID do_handok()
 
 VOID do_ansagen()
 {
+  int i,c,bb;
+
   phase=ANSAGEN;
+  bb=kannspitze=0;
+  for (i=0;i<(handsp?10:12);i++) {
+    c=i>=10?prot2.skat[1][i-10]:cards[spieler*10+i];
+    if ((c&7)==BUBE) bb++;
+    if (i<10) {
+      if ((c&7)==SIEBEN) {
+	kannspitze=1;
+	break;
+      }
+      if (c==BUBE) {
+	kannspitze=2;
+      }
+    }
+  }
+  if (kannspitze==2) {
+    kannspitze=bb!=4;
+  }
   if (!iscomp(spieler) && !ramschspiele) {
     di_spiel();
   }
@@ -959,24 +1070,32 @@ int karobubespielen()
 
 VOID do_angesagt()
 {
-  int sn,c,i;
-
   if (!iscomp(spieler)) {
     remmark(1);
   }
-  phase=SPIELEN;
   stich=1;
   schwz=1;
   nullv=0;
+  spitzeok=0;
   info_spiel();
   sort2[0]=sort2[1]=sort2[2]=trumpf==-1;
+  if (revolang) revolutiondist();
+  if (revolang && numsp && (numsp>1 || iscomp(spieler))) revolutionscr();
+  else spielphase();
+}
+
+VOID spielphase()
+{
+  int sn,c,i;
+
+  phase=SPIELEN;
   sptruempfe=0;
   for (i=0;i<10;i++) {
     c=cards[spieler*10+i];
     if ((c&7)==BUBE || c>>3==trumpf) sptruempfe++;
   }
   karobube();
-  if (ouveang && numsp>1) {
+  if (ouveang) {
     for (sn=0;sn<numsp;sn++) {
       di_info(sn,-2);
       calc_desk(sn);
@@ -1018,16 +1137,13 @@ VOID calc_result()
   int i,c,f;
   int b[4],s[8];
 
-  mes1=mes2=mes3=0;
+  mes1=mes2=mes3=mes4=0;
   if (trumpf==5) {
     ramsch_result();
     return;
   }
   if (trumpf==-1) {
-    f=0;
-    if (handsp) f++;
-    if (ouveang) f+=2;
-    spwert=nullw[f];
+    spwert=nullw[revolang?4:(ouveang?2:0)+handsp];
     if (nullv) {
       spgew=0;
       if (!handsp) spwert*=2;
@@ -1041,6 +1157,11 @@ VOID calc_result()
     if (bockspiele) spwert*=2;
     return;
   }
+  if (stich==1 && schenkstufe) {
+    stsum=61;
+    schwz=0;
+    nullv=1;
+  }
   b[0]=b[1]=b[2]=b[3]=0;
   s[0]=s[1]=s[2]=s[3]=s[4]=s[5]=s[6]=s[7]=0;
   for (i=0;i<12;i++) {
@@ -1048,9 +1169,9 @@ VOID calc_result()
     if ((c&7)==BUBE) b[c>>3]=1;
     else if (c>>3==trumpf) s[c&7]=1;
   }
-  s[BUBE]=s[BUBE+1];
-  s[BUBE+1]=s[BUBE+2];
-  s[BUBE+2]=s[BUBE+3];
+  s[BUBE]=s[NEUN];
+  s[NEUN]=s[ACHT];
+  s[ACHT]=s[SIEBEN];
   f=1;
   while (f<4 && b[3-f]==b[3]) f++;
   if (f==4 && trumpf!=4) {
@@ -1063,11 +1184,15 @@ VOID calc_result()
   if (schwz || schwang || !nullv) f++;
   if (schwang) f++;
   if (ouveang) f++;
+  if (spitzeang) f+=spitzezaehlt;
   if (trumpf==4 && ouveang) spwert=(f-1)*36;
   else spwert=f*rwert[trumpf];
-  if (stsum>60 && spwert>=reizw[reizp]
-    && (stsum>=90 || !schnang)
-    && (schwz || !schwang)) {
+  if ((stsum>60 &&
+       spwert>=reizw[reizp] &&
+       (stsum>=90 || !schnang) &&
+       (schwz || !schwang) &&
+       (spitzeok || !spitzeang))
+      || stich==1) {
     spgew=1;
     nspwert=spwert;
   }
@@ -1075,6 +1200,7 @@ VOID calc_result()
     if (spwert<reizw[reizp]) mes1=1;
     else if (schnang && stsum<90) mes2=1;
     else if (schwang && !schwz) mes3=1;
+    else if (spitzeang && !spitzeok) mes4=1;
     spgew=0;
     if (spwert<reizw[reizp]) {
       spwert=((reizw[reizp]-1)/rwert[trumpf]+1)*rwert[trumpf];
@@ -1092,8 +1218,8 @@ VOID get_next()
 
   prot2.anspiel[stich-1]=ausspl;
   prot2.stiche[stich-1][ausspl]=stcd[0];
-  prot2.stiche[stich-1][(ausspl+1)%3]=stcd[1];
-  prot2.stiche[stich-1][(ausspl+2)%3]=stcd[2];
+  prot2.stiche[stich-1][left(ausspl)]=stcd[1];
+  prot2.stiche[stich-1][right(ausspl)]=stcd[2];
   if (trumpf==-1) null_stich();
   if (higher(stcd[0],stcd[1])) {
     if (higher(stcd[0],stcd[2])) s=0;
@@ -1105,6 +1231,10 @@ VOID get_next()
   }
   ausspl=(ausspl+s)%3;
   prot2.gemacht[stich-1]=ausspl;
+  if (spitzeang && stich==10 && ausspl==spieler &&
+      stcd[s]==(trumpf==4?BUBE:SIEBEN|trumpf<<3)) {
+    spitzeok=1;
+  }
   if (trumpf==5) {
     ramsch_stich();
     return;
@@ -1161,19 +1291,19 @@ int sm[3][3],p,*as,*ae,*ar,*ad;
     sm[s][1]-=e;
     if (!r) {
       sm[s][2]-=50;
-      sm[(s+1)%3][2]+=40;
-      sm[(s+2)%3][2]+=40;
+      sm[left(s)][2]+=40;
+      sm[right(s)][2]+=40;
     }
   }
   else {
     if (r && !d) {
-      sm[(s+1)%3][0]-=e;
-      sm[(s+2)%3][0]-=e;
-      sm[(s+1)%3][2]-=e;
-      sm[(s+2)%3][2]-=e;
+      sm[left(s)][0]-=e;
+      sm[right(s)][0]-=e;
+      sm[left(s)][2]-=e;
+      sm[right(s)][2]-=e;
     }
-    sm[(s+1)%3][1]+=e;
-    sm[(s+2)%3][1]+=e;
+    sm[left(s)][1]+=e;
+    sm[right(s)][1]+=e;
     if (!r && e) {
       sm[s][2]+=50;
     }
@@ -1285,6 +1415,31 @@ VOID read_opt()
       }
     }
   }
+  if (fscanf(f,"s %d\n",&v)==1) {
+    if (!spitzezaehltset) {
+      spitzezaehlt=v;
+    }
+  }
+  if (fscanf(f,"r %d\n",&v)==1) {
+    if (!revolutionset) {
+      revolution=!!v;
+    }
+  }
+  if (fscanf(f,"k %d\n",&v)==1) {
+    if (!klopfenset) {
+      klopfen=!!v;
+    }
+  }
+  if (fscanf(f,"s %d\n",&v)==1) {
+    if (!schenkenset) {
+      schenken=!!v;
+    }
+  }
+  if (fscanf(f,"f %d\n",&v)==1) {
+    if (!fastdealset) {
+      fastdeal=!!v;
+    }
+  }
   fclose(f);
 }
 
@@ -1310,6 +1465,11 @@ VOID save_opt()
   fprintf(f,"t %d %d %d\n",nimmstich[0][0],nimmstich[1][0],nimmstich[2][0]);
   fprintf(f,"l %d %d %d\n",alist[0],alist[1],alist[2]);
   fprintf(f,"m %d %d %d\n",mbutton[0],mbutton[1],mbutton[2]);
+  fprintf(f,"s %d\n",spitzezaehlt);
+  fprintf(f,"r %d\n",revolution);
+  fprintf(f,"k %d\n",klopfen);
+  fprintf(f,"s %d\n",schenken);
+  fprintf(f,"f %d\n",fastdeal);
   fclose(f);
 }
 
@@ -1367,12 +1527,26 @@ VOID read_list()
 
 VOID fill_st()
 {
-  int i,j,s;
+  int i,j,s,c,sc;
 
   for (s=0;s<3;s++) {
+    if (vmh>=1 && s==ausspl) {
+      sc=stcd[0];
+    }
+    else if (vmh==2 && s==left(ausspl)) {
+      sc=stcd[1];
+    }
+    else {
+      sc=-1;
+    }
     i=stich-1;
     for (j=0;j<10;j++) {
-      if (cards[10*s+j]>=0) prot2.stiche[i++][s]=cards[10*s+j];
+      c=cards[10*s+j];
+      if (c<0 && sc>=0) {
+	c=sc;
+	sc=-1;
+      }
+      if (c>=0) prot2.stiche[i++][s]=c;
     }
   }
 }
@@ -1390,6 +1564,8 @@ int maxnimm()
 
 VOID next_stich()
 {
+  int nd;
+
   if (maxnimm()<=100) {
     waitt(maxnimm()*100,2);
   }
@@ -1399,35 +1575,35 @@ VOID next_stich()
   nimm_stich();
   vmh=0;
   stich++;
-  if (stich==11 || (trumpf==-1 && nullv)) {
-    if (stich<11) fill_st();
-    calc_result();
-    prot2.stichgem=stich-1;
-    prot2.spieler=spieler;
-    prot2.trumpf=trumpf;
-    prot2.gereizt=reizp<0?0:reizw[reizp];
-    prot2.gewonn=spgew;
-    prot2.augen=stsum;
-    prot2.ehsso=
-      ouveang?trumpf==-1 && handsp?5:4:schwang?3:schnang?2:handsp?1:0;
-    prot2.predef=predef;
-    if (trumpf!=5) prot2.sramsch=0;
-    prot2.savseed=savseed;
-    prot2.gamenr=gamenr-1;
-    prot2.rotateby=rotateby;
-    prot1=prot2;
-    update_list();
-    if (playbock) bockinc=check_bockevents();
-    save_list();
-    if (logging) di_proto(0,1,1);
-    splres=1;
-    clr_desk();
-    phase=RESULT;
-    di_result(bockinc);
-    nextgame();
+  nd=0;
+  if (stich==11 ||
+      (trumpf==-1 &&
+       (nullv ||
+	(!ndichtw && stich<10 &&
+	 (nd=null_dicht(spieler,handsp,&prot2.skat[1][0],
+			(int *)0,(int *)0,(int *)0)))))) {
+    if (nd) di_dicht();
+    else finishgame();
   }
 }
 
+VOID finishgame()
+{
+  if (stich<11) fill_st();
+  calc_result();
+  set_prot();
+  prot1=prot2;
+  update_list();
+  if (playbock) bockinc=check_bockevents();
+  save_list();
+  if (logging) di_proto(0,1,1);
+  splres=1;
+  clr_desk(0);
+  phase=RESULT;
+  di_result(bockinc);
+  nextgame();
+}
+
 VOID do_next()
 {
   int sn;
@@ -1535,14 +1711,24 @@ int ci;
 
 int ignorieren()
 {
-  int mi,fb,i,ih;
+  int mi,fb,i,ih,k[8];
 
-  mi=(ausspl+2)%3;
+  mi=right(ausspl);
   fb=stcd[0]>>3;
   if ((stcd[0]&7)==BUBE) fb=trumpf;
   ih=0;
+  for (i=0;i<8;i++) k[i]=0;
   for (i=0;i<possc;i++) {
-    if (cards[possi[i]]>>3==fb) ih++;
+    if (cards[possi[i]]>>3==fb) {
+      ih++;
+      k[cards[possi[i]]&7]=1;
+    }
+  }
+  if (ih==2) {
+    k[BUBE]=k[NEUN];
+    for (i=AS;i<=DAME;i++) {
+      if (k[i] && k[i+1]) return 0;
+    }
   }
   return fb!=trumpf && !cardw[stcd[0]&7] && hatnfb[mi][fb]!=1 && ih<3;
 }
@@ -1714,10 +1900,10 @@ int ueberdoerfer()
 
 int trumpfausspielen()
 {
-  int i,j,g1,g2,tr,trdr,wi,wj;
+  int i,j,k,g1,g2,tr,trdr,wi,wj;
 
-  g1=(spieler+1)%3;
-  g2=(spieler+2)%3;
+  g1=left(spieler);
+  g2=right(spieler);
   if (!hatnfb[g1][trumpf] ||
     !hatnfb[g2][trumpf]) {
     if (niedrighoch(trumpf)) return 1;
@@ -1736,11 +1922,34 @@ int trumpfausspielen()
       }
     }
   }
-  if (trumpf<4) trdr=7-gespfb[trumpf];
+  k=possc;
+  if (trumpf<4) {
+    trdr=7-gespfb[trumpf];
+    if (wj!=-1 && (hatnfb[g1][trumpf] || hatnfb[g2][trumpf])) {
+      calc_inhand(spieler);
+      for (i=SIEBEN;i>=DAME;i--) {
+	if (i==BUBE) continue;
+	if (!gespcd[trumpf<<3|i] && !inhand[trumpf][i]) {
+	  for (;i>=KOENIG;i--) {
+	    if (i==BUBE) continue;
+	    if (inhand[trumpf][i]) {
+	      for (k=0;k<possc;k++) {
+		if (cards[possi[k]]==(trumpf<<3|i)) {
+		  break;
+		}
+	      }
+	      break;
+	    }
+	  }
+	  break;
+	}
+      }
+    }
+  }
   else trdr=0;
   for (i=0;i<4;i++) if (!gespcd[i<<3|BUBE]) trdr++;
   if ((tr>2 && (trumpf!=4 || trdr-tr)) || (tr>1 && trdr-tr && trdr-tr<=2)) {
-    playcd=j;
+    playcd=k!=possc && (trdr-tr==2 || !cardw[cards[possi[k]]&7])?k:j;
     return 1;
   }
   for (i=0;i<possc;i++) {
@@ -1774,7 +1983,7 @@ int hochausspielen()
   return 0;
 }
 
-VOID schenken()
+VOID schenke()
 {
   int i,j,ci,cj,wi,wj;
 
@@ -1877,7 +2086,7 @@ int f;
 	  if ((cj&7)!=BUBE && hatnfb[spieler][cj>>3]) wj-=30;
 	}
 	else {
-	  mi=spieler==(ausspl+1)%3?2:1;
+	  mi=spieler==left(ausspl)?2:1;
 	  wio=wi;
 	  wjo=wj;
 	  if (!hatnfb[spieler][ci>>3] || wi>4) wi+=8;
@@ -1897,7 +2106,7 @@ int f;
             }
           }
           if (n[ci>>3]<n[cj>>3]) wi=0;
-          if (spieler==(ausspl+1)%3) swap(&wi,&wj);
+          if (spieler==left(ausspl)) swap(&wi,&wj);
         }
       }
       else {
@@ -1923,8 +2132,8 @@ int buttern()
 {
   int fb,mi,se;
 
-  se=(ausspl+1)%3;
-  mi=spieler==ausspl?(ausspl+2)%3:ausspl;
+  se=left(ausspl);
+  mi=spieler==ausspl?right(ausspl):ausspl;
   fb=stcd[0]>>3;
   if ((stcd[0]&7)==BUBE) fb=trumpf;
   if (!hatnfb[se][fb]) return 0;
@@ -1987,31 +2196,49 @@ int f;
     if (cards[possi[i]]<cards[possi[j]]) j=i;
     if (cards[possi[i]]>cards[possi[k]]) k=i;
   }
-  hi=f?higher(stcd[0],stcd[1])^spieler==ausspl:cards[possi[j]]<stcd[0];
+  hi=f?higher(stcd[0],stcd[1])^(spieler==ausspl):cards[possi[j]]<stcd[0];
   playcd=hi?j:k;
   return 1;
 }
 
+VOID nichtspitze()
+{
+  int sp,i;
+
+  if (spitzeang) {
+    sp=trumpf==4?BUBE:SIEBEN|trumpf<<3;
+    for (i=0;i<possc;i++) {
+      if (cards[possi[i]]==sp) {
+	possc--;
+	for (;i<possc;i++) {
+	  possi[i]=possi[i+1];
+	}
+	return;
+      }
+    }
+  }
+}
+
 VOID m_bvsp()
 {
   if (ueberdoerfer()) return;
-  if (!trumpfausspielen()) schenken();
+  if (!trumpfausspielen()) schenke();
 }
 
 VOID m_bmsp()
 {
-  if (!uebernehmen(0,0)) schenken();
+  if (!uebernehmen(0,0)) schenke();
 }
 
 VOID m_bhsp()
 {
   if (fabwerfen()) return;
-  if (!uebernehmen(!higher(stcd[0],stcd[1]),1)) schenken();
+  if (!uebernehmen(!higher(stcd[0],stcd[1]),1)) schenke();
 }
 
 VOID m_bvns()
 {
-  if (spieler==(ausspl+1)%3 && karobubespielen()) return;
+  if (spieler==left(ausspl) && karobubespielen()) return;
   if (!hochausspielen()) abwerfen(1);
 }
 
@@ -2032,7 +2259,7 @@ VOID m_bhns()
 {
   if (karobubespielen()) return;
   if (schnippeln(1)) return;
-  if (higher(stcd[0],stcd[1])^spieler!=ausspl) {
+  if (higher(stcd[0],stcd[1])^(spieler!=ausspl)) {
     if (!uebernehmen(spieler!=ausspl,1)) abwerfen(0);
   }
   else {
@@ -2043,6 +2270,7 @@ VOID m_bhns()
 VOID m_bsp()
 {
   playcd=0;
+  nichtspitze();
   if (!vmh) m_bvsp();
   else if (vmh==1) m_bmsp();
   else m_bhsp();
@@ -2078,12 +2306,13 @@ int s;
 VOID adjfb(s,v)
 int s,v;
 {
-  int i,c;
+  int i,c,n;
   int fb[5];
 
   fb[0]=fb[1]=fb[2]=fb[3]=fb[4]=0;
-  for (i=0;i<10;i++) {
-    if ((c=cards[10*s+i])>=0) {
+  n=handsp && s!=spieler?12:10;
+  for (i=0;i<n;i++) {
+    if ((c=i<10?cards[10*s+i]:prot2.skat[0][i-10])>=0) {
       if (trumpf!=-1 && (c&7)==BUBE) fb[trumpf]=1;
       else fb[c>>3]=1;
     }
@@ -2097,13 +2326,19 @@ int s,v;
 
 VOID do_spielen()
 {
-  int s;
+  int s,i;
 
   if (trumpf==-1 && stich==1) init_null();
   while (phase==SPIELEN && iscomp(s=(ausspl+vmh)%3)) {
     if (s==spieler && trumpf!=5) {
-      adjfb((spieler+1)%3,2);
-      adjfb((spieler+2)%3,2);
+      adjfb(left(spieler),2);
+      adjfb(right(spieler),2);
+      for (i=0;i<5;i++) {
+	if (!hatnfb[left(spieler)][i] || !hatnfb[right(spieler)][i]) {
+	  if (hatnfb[left(spieler)][i]==2) hatnfb[left(spieler)][i]=0;
+	  if (hatnfb[right(spieler)][i]==2) hatnfb[right(spieler)][i]=0;
+	}
+      }
     }
     if (ouveang) {
       adjfb(spieler,1);
