@@ -42,8 +42,12 @@ enum {
  HANDSPIEL,
  DRUECKEN,
  ANSAGEN,
+ REVOLUTION,
  SPIELEN,
+ SCHENKEN,
  NIMMSTICH,
+ NULLDICHT,
+ WEITER,
  RESULT
 };
 
@@ -51,7 +55,7 @@ EXTERN int nullw[]
 #ifdef SKAT_C
 =
 {
-  23,35,46,59
+  23,35,46,59,92
 }
 #endif
 ;
@@ -110,32 +114,38 @@ EXTERN int playkontra,kontraset,kontrastufe;
 EXTERN int playbock,bockset,bockevents,bockeventsset;
 EXTERN int bockspiele,ramschspiele,bockinc;
 EXTERN int resumebock,resumebockset;
+EXTERN int spitzezaehlt,spitzezaehltset;
+EXTERN int revolution,revolutionset;
+EXTERN int klopfen,klopfenset;
+EXTERN int schenken,schenkenset,schenkstufe;
+EXTERN int fastdeal,fastdealset;
 EXTERN int sagte18[3];
 EXTERN int cards[32],gespcd[32];
 EXTERN int quit,phase,geber,hoerer,sager,spieler;
 EXTERN int saho,reizp,gedr,vmh,stich,ausspl;
 EXTERN int possi[10],possc,stcd[3];
-EXTERN int trumpf,handsp,stsum,spcards[12];
-EXTERN int spgew,spwert,schwz,nullv,nspwert;
-EXTERN int schnang,schwang,ouveang;
+EXTERN int trumpf,handsp,stsum,spcards[12],kannspitze;
+EXTERN int spgew,spwert,schwz,nullv,spitzeok,nspwert;
+EXTERN int schnang,schwang,ouveang,spitzeang,revolang;
 EXTERN struct { int s,r,d,e,g; } splist[360];
 EXTERN int splstp,splres;
 EXTERN int gespfb[4],high[5],shigh[5];
 EXTERN int inhand[4][8];
 EXTERN int playcd,drkcd;
-EXTERN int mes1,mes2,mes3;
+EXTERN int mes1,mes2,mes3,mes4;
 EXTERN int butternok,sptruempfe;
 EXTERN int predef,logging,unformatted;
 EXTERN long seed[2],savseed,gamenr,rotateby;
 EXTERN char *list_file,*game_file,*prot_file,*opt_file;
 EXTERN int wieder,vorhandwn;
 EXTERN int karobubeanz;
+EXTERN int ndichtw;
 EXTERN struct
 {
   int stichgem,spieler,trumpf,gereizt,gewonn,augen,ehsso,predef,sramsch;
   int stiche[10][3],anspiel[10],gemacht[10],skat[4][2],verdopp[3];
   long savseed,gamenr;
-  int rotateby;
+  int rotateby,spitze,revolution,schenken,handsp;
 } prot1,prot2;
 
 #endif /* SKAT_H */
