@@ -15,11 +15,9 @@
 #include "defs.h"
 #include "text.h"
 
-#define VERSION   " X S K A T   2.2 "
+#define VERSION   " X S K A T   2.3.1 "
 #define COPYRIGHT "Copyright 1998  Gunter Gerhardt"
 #define EMAIL     "(gerhardt@draeger.com)"
-#define HOMEPAGE  "Xskat Home Page"
-#define URL       "www.draeger.com/~gerhardt/xskat.html"
 
 static char *ger_text[]={
   "Null","Karo","Herz","Pik","Kreuz","Grand","Ramsch",
@@ -45,6 +43,7 @@ static char *ger_text[]={
   "Wert des angesagten Spiels !",
   "Null:23  Null ouvert:46",
   "Null Hand:35  Null ouvert Hand:59",
+  "Revolution:92",
   " Angesagt ist "," Resultat ","Der Spielwert ist","Ende",
   "Protokoll","Weiter"," Protokoll ",
   "Im Skat ist:",
@@ -53,11 +52,11 @@ static char *ger_text[]={
   "Gewonnen mit:",
   "Verloren mit:",
   "Spielliste"," Spielliste ",VERSION,
-  COPYRIGHT,EMAIL,HOMEPAGE,URL,
+  COPYRIGHT,EMAIL,
   "Sortieren","Aufwrts","Abwrts","Alternierend","Sequentiell",
   "Normal",
   "Gereizt bis:","Gespielt wird:","Letzter Stich:","Gedrckt:",
-  "Du bist dran!","Vordefiniertes Spiel","Speichern",
+  "Du bist dran !","Vordefiniertes Spiel","Speichern",
   "Dieses Programm ist freie Software;",
   "es kann frei verbreitet werden.",
   "Verwendung auf eigenes Risiko;",
@@ -67,9 +66,9 @@ static char *ger_text[]={
   "Ramsch spielen","Immer",
   "Schieberamsch"," Skat aufnehmen ? ","Fertig",
   "Buben drfen nicht geschoben werden.",
-  " Spielwert verdoppelt ","nimmt den Skat nicht auf.",
+  " Spielwert verdoppelt ","nimmt den Skat nicht auf.","klopft !",
   "VH schob:","MH schob:","HH schob:",
-  "Kontra sagen","mit Kontra","ab 18","Kontra"," Kontra! ","Re","von ",
+  "Kontra sagen","mit Kontra","ab 18","Kontra"," Kontra ! ","Re","von ",
   "Wiederholen"," Spiel wiederholen ","mit den Karten von","mir",
   "Vorhand wechselt:",
   " Verzgerung ","Nimm Stich nach:","Sekunden","Maus-Klick",
@@ -80,7 +79,18 @@ static char *ger_text[]={
   "Erfolgreicher Kontra","Kontra & Re angesagt",
   "NNN Punkte in Spielliste","N00 Punkte in Spielliste",
   "Spielwert ist >= +72","Spielwert ist >= +96",
-  " Men-Button ","Jeder"
+  " Men-Button ","Jeder",
+  "Spitze","zhlt 2","Spitze verloren !","Spitze !",
+  "Spitze nicht erlaubt !",
+  "Niedrigster Trumpf nicht auf der Hand.",
+  "Grand mit 4 Buben.",
+  "Null ist dicht !"," Null ist dicht ! ","Karten aufdecken ?",
+  "Revolution","Revolution: Karten austauschen !",
+  "Vorhand","Mittelhand","Hinterhand",
+  "Klopfen"," Klopfen ","Spielwert verdoppeln ?",
+  "Schenken"," Schenken ","Spiel verloren geben ?","Geschenk annehmen ?",
+  "Mitspieler lehnt ab.","Gegner geben auf.","Annehmen",
+  "Geben","Schnell","Langsam"
 };
 
 static char *eng_text[]={
@@ -107,6 +117,7 @@ static char *eng_text[]={
   "is not high enough !",
   "Null:23  Null ouvert:46",
   "Null Hand:35  Null ouvert Hand:59",
+  "Revolution:92",
   " Playing "," Result ","The value of the game is","Quit",
   "Log","Continue"," Log ",
   "Skat is:",
@@ -115,11 +126,11 @@ static char *eng_text[]={
   "Won with:",
   "Lost with:",
   "Game list"," Game list ",VERSION,
-  COPYRIGHT,EMAIL,HOMEPAGE,URL,
+  COPYRIGHT,EMAIL,
   "Sort","Up","Down","Alternating","Sequential",
   "Normal",
   "Bidding:","Playing:","Last trick:","Skat:",
-  "It's your turn!","predefined game","Save",
+  "It's your turn !","predefined game","Save",
   "This program is free software;",
   "you can redistribute it freely.",
   "Use it at your own risk;",
@@ -129,9 +140,9 @@ static char *eng_text[]={
   "Play Ramsch","Always",
   "Schieberamsch"," Pick up Skat ? ","Done",
   "It's not allowed to pass on jacks.",
-  " Game score doubled ","doesn't pick up the Skat.",
+  " Game score doubled ","doesn't pick up the Skat.","knocks !",
   "FH passed:","MH passed:","RH passed:",
-  "Say Kontra","with Kontra","if 18","Kontra"," Kontra! ","Re","by ",
+  "Say Kontra","with Kontra","if 18","Kontra"," Kontra ! ","Re","by ",
   "Replay"," Replay game ","with the cards from","me",
   "Forehand changes:",
   " Delay ","Take trick after:","seconds","mouse click",
@@ -142,7 +153,18 @@ static char *eng_text[]={
   "Successful Kontra","Kontra & Re game",
   "NNN points in game list","N00 points in game list",
   "Game value is >= +72","Game value is >= +96",
-  " Menu button ","Any"
+  " Menu button ","Any",
+  "Spitze","counts 2","Lost Spitze !","Spitze !",
+  "Spitze not allowed !",
+  "Lowest trump not in your hand.",
+  "Grand with 4 jacks.",
+  "Null is safe !"," Null is safe ! ","Show your cards ?",
+  "Revolution","Revolution: exchange cards !",
+  "Forehand","Middlehand","Rearhand",
+  "Klopfen"," Klopfen ","Double game value ?",
+  "Schenken"," Schenken ","Give up the game ?","Accept the gift ?",
+  "Partner disagrees.","Opponents give up.","Accept",
+  "Deal","Fast","Slow"
 };
 
 static struct {
