@@ -41,6 +41,7 @@ enum {
   TX_U2,
   TX_U3,
   TX_U4,
+  TX_U5,
   TX_ANGESAGT_IST,TX_RESULTAT,TX_DER_SPIELWERT_IST,TX_ENDE,
   TX_PROTOKOLL,TX_WEITER,TX_PROTOKOLL_T,
   TX_IM_SKAT_IST,
@@ -49,7 +50,7 @@ enum {
   TX_GEWO_MIT,
   TX_VERL_MIT,
   TX_SPIELLISTE,TX_SPIELLISTE_T,TX_XSKAT,
-  TX_COPYRIGHT,TX_EMAIL,TX_HOMEPAGE,TX_URL,
+  TX_COPYRIGHT,TX_EMAIL,
   TX_SORTIEREN,TX_AUFWAERTS,TX_ABWAERTS,TX_ALTERNIEREND,TX_SEQUENTIELL,
   TX_NORMAL,
   TX_GEREIZT_BIS_L,TX_GESPIELT_WIRD,TX_LETZTER_STICH,TX_GEDRUECKT,
@@ -63,7 +64,7 @@ enum {
   TX_RAMSCH_SPIELEN,TX_IMMER,
   TX_SCHIEBERAMSCH,TX_SKAT_AUFNEHMEN,TX_FERTIG,
   TX_BUBEN_NICHT,
-  TX_VERDOPPELT,TX_SCHIEBT,
+  TX_VERDOPPELT,TX_SCHIEBT,TX_KLOPFT,
   TX_VH_SCHOB,TX_MH_SCHOB,TX_HH_SCHOB,
   TX_KONTRA_SAGEN,TX_MIT_KONTRA,TX_AB18,TX_KONTRA,TX_KONTRA_T,TX_RE,TX_VON,
   TX_WIEDER,TX_SPIEL_WIEDER,TX_MIT_DEN_KARTEN,TX_MIR,
@@ -77,6 +78,17 @@ enum {
   TX_NNN_IN_SPIELLISTE,TX_N00_IN_SPIELLISTE,
   TX_SPIELWERT_72,TX_SPIELWERT_96,
   TX_MENU_BUTTON,TX_JEDER,
+  TX_SPITZE,TX_PLUS2,TX_SPITZE_VERLOREN,TX_SPITZE_PROT,
+  TX_SPITZE_F1,
+  TX_SPITZE_F2,
+  TX_SPITZE_F3,
+  TX_NULL_DICHT,TX_NULL_DICHT_T,TX_KARTEN_AUFDECKEN,
+  TX_REVOLUTION,TX_KARTEN_AUSTAUSCHEN,
+  TX_VORHAND,TX_MITTELHAND,TX_HINTERHAND,
+  TX_KLOPFEN,TX_KLOPFEN_T,TX_SPIELWERT_VERDOPPELN,
+  TX_SCHENKEN,TX_SCHENKEN_T,TX_SPIEL_VERLOREN_GEBEN,TX_GESCHENK_ANNEHMEN,
+  TX_MITSPIELER_LEHNT_AB,TX_GEGNER_GEBEN_AUF,TX_AKZEPTIERT,
+  TX_GEBEN,TX_SCHNELL,TX_LANGSAM,
   TX_LAST
 };
 
