@@ -55,7 +55,7 @@ VOID info_spiel()
   for (sn=0;sn<numsp;sn++) {
     dioptions[sn][12].str=txt[sn];
     sprintf(txt[sn],"%s %s",textarr[TX_GESPIELT_WIRD],
-	    textarr[TX_NULL+trumpf+1]);
+	    textarr[revolang?TX_REVOLUTION:TX_NULL+trumpf+1]);
   }
 }
 
@@ -67,13 +67,12 @@ int p,c,f;
 
   for (sn=0;sn<numsp;sn++) {
     if (!f || sn==spieler) {
-      dioptions[sn][15+p].str=txt[sn][p];
+      dioptions[sn][14+p].str=txt[sn][p];
       txt[sn][p][0]=0;
       strcat(txt[sn][p],textarr[TX_KARO+(c>>3)]);
       strcat(txt[sn][p]," ");
       strcat(txt[sn][p],textarr[TX_AS+(c&7)]);
-      dioptions[sn][13].spec=f?OB_HIDDEN:OB_NONE;
-      dioptions[sn][14].spec=f?OB_NONE:OB_HIDDEN;
+      dioptions[sn][13].str=textarr[f?TX_GEDRUECKT:TX_LETZTER_STICH];
     }
   }
 }
@@ -85,11 +84,10 @@ VOID clear_info()
   for (sn=0;sn<numsp;sn++) {
     dioptions[sn][11].str=textarr[TX_GEREIZT_BIS_L];
     dioptions[sn][12].str=textarr[TX_GESPIELT_WIRD];
-    dioptions[sn][13].spec=OB_NONE;
-    dioptions[sn][14].spec=OB_HIDDEN;
+    dioptions[sn][13].str=textarr[TX_LETZTER_STICH];
+    dioptions[sn][14].str=OB_NONE;
     dioptions[sn][15].str=OB_NONE;
     dioptions[sn][16].str=OB_NONE;
-    dioptions[sn][17].str=OB_NONE;
   }
 }
 
@@ -152,14 +150,23 @@ VOID init_dials()
   init_di(dischieben);
   init_di(diverdoppelt);
   init_di(dibuben);
+  init_di(diklopfen);
+  init_di(dischenken);
+  init_di(digeschenkt);
+  init_di(diendeschenken);
+  init_di(diwiederschenken);
+  init_di(dinichtschenken);
   init_di(diloesch);
   init_di(dispiel);
   init_di(difehler);
   init_di(diueberr);
+  init_di(dispitze);
   init_di(diansage);
   init_di(dikontra);
   init_di(direkontra);
   init_di(dikonre);
+  init_di(didicht);
+  init_di(diweiter);
   init_di(diresult);
   init_di(diwieder);
   for (n=-4;n<=4;n++) {
@@ -197,7 +204,7 @@ VOID hndl_btevent(sn,event)
 int sn;
 XEvent *event;
 {
-  int bt,i,j,ag,s1,s2,al,stg[2],sav;
+  int bt,i,j,c,ag,s1,s2,al,stg[2],sav,bb;
   OBJECT *ob;
 
   ob=actdial[sn];
@@ -266,7 +273,8 @@ XEvent *event;
 	else if (ob==dispiel) {
 	  remove_di(sn);
 	  ag=0;
-	  if (dispiel[6].spec&OB_SELECTED) trumpf=-1;
+	  if (dispiel[6].spec&OB_SELECTED ||
+	      dispiel[13].spec&OB_SELECTED) trumpf=-1;
 	  else if (dispiel[2].spec&OB_SELECTED) trumpf=0;
 	  else if (dispiel[3].spec&OB_SELECTED) trumpf=1;
 	  else if (dispiel[4].spec&OB_SELECTED) trumpf=2;
@@ -284,12 +292,31 @@ XEvent *event;
 	    ag=1;
 	  }
 	  if (!ag && trumpf==-1 &&
-	      reizw[reizp]>nullw[handsp+(dispiel[10].spec&OB_SELECTED?2:0)]) {
+	      reizw[reizp]>nullw[dispiel[13].spec&OB_SELECTED?4:
+				 (dispiel[10].spec&OB_SELECTED?2:0)+handsp]) {
+	    diueberr[6].spec=revolution?OB_CENTERED:OB_HIDDEN;
 	    create_di(sn,diueberr);
 	    ag=1;
 	  }
+	  spitzeang=0;
+	  if (!ag && trumpf!=-1 && dispiel[11].spec&OB_SELECTED) {
+	    bb=0;
+	    for (i=0;i<(handsp?10:12);i++) {
+	      c=i>=10?prot2.skat[1][i-10]:cards[spieler*10+i];
+	      if (i<10 && c==(trumpf==4?BUBE:SIEBEN|trumpf<<3)) {
+		spitzeang=1;
+	      }
+	      if ((c&7)==BUBE) bb++;
+	    }
+	    if (!spitzeang || (bb==4 && trumpf==4)) {
+	      dispitze[3].str=textarr[spitzeang?TX_SPITZE_F3:TX_SPITZE_F2];
+	      create_di(sn,dispitze);
+	      ag=1;
+	    }
+	  }
 	  if (!ag) {
-	    if (dispiel[10].spec&OB_SELECTED) ouveang=schwang=schnang=1;
+	    if (dispiel[13].spec&OB_SELECTED) revolang=ouveang=1;
+	    else if (dispiel[10].spec&OB_SELECTED) ouveang=schwang=schnang=1;
 	    else if (dispiel[9].spec&OB_SELECTED) schwang=schnang=1;
 	    else if (dispiel[8].spec&OB_SELECTED) schnang=1;
 	    if (trumpf==-1) schwang=schnang=0;
@@ -304,52 +331,21 @@ XEvent *event;
 	  remove_di(sn);
 	  di_spiel();
 	}
+	else if (ob==dispitze) {
+	  remove_di(sn);
+	  di_spiel();
+	}
 	else if (ob==diansage) {
 	  remove_di(sn);
 	  do_angesagt();
 	}
 	else if (ob==dikontra) {
 	  remove_di(sn);
-	  if (bt==3) {
-	    kontrastufe=1;
-	    if (!iscomp(spieler)) {
-	      di_rekontra(sn);
-	    }
-	    else if (numsp-!iscomp(spieler)>1) {
-	      ktrnext=-1;
-	      di_konre(((sn+1)%3==spieler?sn+2:sn+1)%3);
-	    }
-	    else {
-	      do_angesagt();
-	    }
-	  }
-	  else if (ktrnext>=0 && ktrnext!=sn) {
-	    di_kontra(ktrnext);
-	    ktrnext=-1;
-	  }
-	  else {
-	    do_angesagt();
-	  }
+	  di_ktrnext(sn,bt==3);
 	}
 	else if (ob==direkontra) {
 	  remove_di(sn);
-	  if (bt==3) {
-	    kontrastufe=2;
-	    if (numsp-!iscomp(spieler)>1) {
-	      ktrnext=((ktrsag+1)%3==spieler?ktrsag+2:ktrsag+1)%3;
-	    }
-	    else {
-	      ktrnext=-1;
-	    }
-	    di_konre(ktrsag);
-	  }
-	  else if (numsp-!iscomp(spieler)>1) {
-	    ktrnext=-1;
-	    di_konre(((ktrsag+1)%3==spieler?ktrsag+2:ktrsag+1)%3);
-	  }
-	  else {
-	    do_angesagt();
-	  }
+	  di_ktrnext(sn,bt==3);
 	}
 	else if (ob==dikonre) {
 	  remove_di(sn);
@@ -361,19 +357,90 @@ XEvent *event;
 	    do_angesagt();
 	  }
 	}
+	else if (ob==didicht) {
+	  remove_di(sn);
+	  if (didicht[3].spec&OB_SELECTED) {
+	    nullendscr();
+	  }
+	  else {
+	    ndichtw=1;
+	    phase=SPIELEN;
+	  }
+	}
+	else if (ob==diweiter) {
+	  remove_di(sn);
+	  di_weiter(0);
+	}
 	else if (ob==dischieben) {
 	  remove_di(sn);
 	  if (bt==2) {
 	    draw_skat();
-	    put_fbox(TX_FERTIG);
+	    put_fbox(spieler,TX_FERTIG);
 	  }
 	  else {
-	    di_verdoppelt(0);
+	    di_verdoppelt(0,0);
 	  }
 	}
+	else if (ob==diklopfen) {
+	  remove_di(sn);
+	  if (bt==3) {
+	    vmh=left(vmh);
+	    if (vmh) di_schieben();
+	    else start_ramsch();
+	  }
+	  else {
+	    di_verdoppelt(0,1);
+	  }
+	}
+	else if (ob==dischenken) {
+	  remove_di(sn);
+	  if (bt==3) {
+	    di_nichtschenken(sn);
+	  }
+	  else {
+	    di_geschenkt();
+	  }
+	}
+	else if (ob==digeschenkt || ob==diendeschenken) {
+	  remove_di(sn);
+	  if (bt==4) {
+	    schenkstufe++;
+	    if (schnang && schenkstufe==2) schenkstufe++;
+	    if (schenkstufe<3) schnang=1;
+	    else schwang=1;
+	    diwiederschenken[2].str=
+	      textarr[schwang?TX_SCHW_ANGE:TX_SCHN_ANGE];
+	    di_wiederschenken(schenknext,1);
+	  }
+	  else {
+	    finishgame();
+	  }
+	}
+	else if (ob==diwiederschenken) {
+	  remove_di(sn);
+	  if (bt==4) {
+	    di_nichtschenken(sn);
+	  }
+	  else {
+	    if (schenknext!=-1) {
+	      di_wiederschenken(schenknext,0);
+	    }
+	    else {
+	      schenknext=sn;
+	      sn=left(sn)==spieler?left(spieler):left(sn);
+	      if (!iscomp(sn)) schenknext=sn;
+	      di_geschenkt();
+	    }
+	  }
+	}
+	else if (ob==dinichtschenken) {
+	  remove_di(sn);
+	  if (schenkstufe==1) schenkstufe=0;
+	  phase=SPIELEN;
+	}
 	else if (ob==diverdoppelt) {
 	  remove_di(sn);
-	  di_verdoppelt(1);
+	  di_verdoppelt(1,0);
 	}
 	else if (ob==dibuben) {
 	  remove_di(sn);
@@ -459,6 +526,7 @@ XEvent *event;
 	    if (numsp==3) di_varianten(sn);
 	    else di_strateg(sn);
 	  }
+	  else if (bt==17) di_schenken(sn);
 	  else if (bt==2) di_copyr(sn);
 	}
 	else if (ob==dicopyr[sn]) {
@@ -488,11 +556,15 @@ XEvent *event;
 	}
 	else if (ob==divarianten[sn]) {
 	  remove_di(sn);
-	  sav=(((((playramsch)*3
-		  +playkontra)*3
-		 +playsramsch)*3
-		+playbock)*3
-	       +resumebock);
+	  sav=((((((((playramsch
+		      )*3+playkontra
+		     )*3+playbock
+		    )*3+spitzezaehlt
+		   )*2+playsramsch
+		  )*2+resumebock
+		 )*2+revolution
+		)*2+klopfen
+	       )*2+schenken;
 	  for (i=0;i<3;i++) {
 	    if (divarianten[sn][3+i].spec&OB_SELECTED) {
 	      playramsch=i;
@@ -503,14 +575,24 @@ XEvent *event;
 	    if (divarianten[sn][14+i].spec&OB_SELECTED) {
 	      playbock=i;
 	    }
+	    if (divarianten[sn][21+i].spec&OB_SELECTED) {
+	      spitzezaehlt=i;
+	    }
 	  }
 	  playsramsch=!(divarianten[sn][7].spec&OB_SELECTED);
 	  resumebock=!(divarianten[sn][18].spec&OB_SELECTED);
-	  if (sav!=(((((playramsch)*3
-		       +playkontra)*3
-		      +playsramsch)*3
-		     +playbock)*3
-		    +resumebock)) {
+	  revolution=!(divarianten[sn][25].spec&OB_SELECTED);
+	  klopfen=!(divarianten[sn][28].spec&OB_SELECTED);
+	  schenken=!(divarianten[sn][31].spec&OB_SELECTED);
+	  if (sav!=((((((((playramsch
+			   )*3+playkontra
+			  )*3+playbock
+			 )*3+spitzezaehlt
+			)*2+playsramsch
+		       )*2+resumebock
+		      )*2+revolution
+		     )*2+klopfen
+		    )*2+schenken) {
 	    save_opt();
 	  }
 	  if (playbock) di_bockevents(sn);
@@ -529,7 +611,7 @@ XEvent *event;
 	  di_verzoegerung(sn);
 	}
 	else if (ob==diverzoegerung[sn]) {
-	  sav=nimmstich[0][0];
+	  sav=nimmstich[0][0]*2+fastdeal;
 	  diverzoegerung[sn][bt].spec&=~OB_SELECTED;
 	  switch (bt) {
 	  case 4:
@@ -542,21 +624,22 @@ XEvent *event;
 	  case 6:
 	    nimmstich[sn][0]--;
 	    break;
-	  case 8:
+	  case 7:
 	    nimmstich[sn][0]++;
 	    break;
-	  case 9:
+	  case 8:
 	    nimmstich[sn][0]+=10;
 	    break;
-	  case 10:
+	  case 9:
 	    nimmstich[sn][0]=101;
 	    break;
 	  }
 	  if (nimmstich[sn][0]<0) nimmstich[sn][0]=0;
 	  else if (nimmstich[sn][0]>101) nimmstich[sn][0]=101;
-	  if (sav!=nimmstich[0][0]) save_opt();
+	  fastdeal=diverzoegerung[sn][11].spec&OB_SELECTED?1:0;
+	  if (sav!=nimmstich[0][0]*2+fastdeal) save_opt();
 	  prverz(sn);
-	  if (bt==7) {
+	  if (bt==13) {
 	    remove_di(sn);
 	    di_menubutton(sn);
 	  }
@@ -676,6 +759,7 @@ OBJECT *ob;
 {
   int x,y;
 
+  if (actdial[sn]) remove_di(sn);
   x=(desk[sn].w-ob[0].w*charw[sn])/2;
   y=(desk[sn].h-ob[0].h*charh[sn])/2;
   create_dial(sn,x,y,0,ob);
@@ -716,7 +800,7 @@ OBJECT *ob;
   else {
     if (ob[i].spec&OB_HIDDEN) return;
     XClearWindow(dpy[sn],ob[i].win);
-    if (ob[i].str && (l=strlen(ob[i].str))) {
+    if (ob[i].str!=OB_NONE && (l=strlen(ob[i].str))) {
       x=y=0;
       if (ob[i].spec&(OB_BUTTON|OB_EXIT|OB_CENTERED)) {
 	x=(ob[i].w*charw[sn]-XTextWidth(dfont[sn],ob[i].str,l))/2;
@@ -821,48 +905,49 @@ int sn,th;
   for (s=0;s<3;s++) {
     if (s!=sn) {
       prspnam(txt,s);
-      w=20*charw[sn];
       x=(s==left(sn)?desk[sn].com1x
-	 :desk[sn].com2x)+30*desk[sn].f/desk[sn].q-10*charw[sn];
+	 :desk[sn].com2x)+30*desk[sn].f/desk[sn].q;
       y=(s==left(sn)?desk[sn].com1y
 	 :desk[sn].com2y)+130*desk[sn].f/desk[sn].q;
       if (th<-1) {
-	v_gtext(sn,x,y,w,clr);
+	v_gtext(sn,x,y,0,clr);
 	continue;
       }
-      v_gtext(sn,x,y,w,txt);
+      v_gtext(sn,x,y,0,txt);
       y+=charh[sn];
-      if (phase==SPIELEN && stich==1 && s==spieler && trumpf!=5) {
-	  v_gtext(sn,x,y,w,clr);
-	  strcpy(txt,textarr[TX_SPIELT]);
-	  strcat(txt,textarr[TX_NULL+trumpf+1]);
-	  v_gtext(sn,x,y,w,txt);
-	  y+=charh[sn];
-	  if (ouveang) {
-	    if (handsp && trumpf==-1) {
-	      v_gtext(sn,x,y,w,textarr[TX_OUVE_HAND]);
-	    }
-	    else {
-	      v_gtext(sn,x,y,w,textarr[TX_OUVE]);
-	    }
+      if ((phase==SPIELEN || phase==SCHENKEN) &&
+	  stich==1 && s==spieler && trumpf!=5) {
+	v_gtext(sn,x,y,0,clr);
+	strcpy(txt,textarr[TX_SPIELT]);
+	strcat(txt,textarr[revolang?TX_REVOLUTION:TX_NULL+trumpf+1]);
+	v_gtext(sn,x,y,0,txt);
+	y+=charh[sn];
+	v_gtext(sn,x,y,0,clr);
+	if (ouveang && !revolang) {
+	  if (handsp && trumpf==-1) {
+	    v_gtext(sn,x,y,0,textarr[TX_OUVE_HAND]);
 	  }
-	  else if (schwang) {
-	    v_gtext(sn,x,y,w,textarr[TX_SCHW_ANGE]);
-	  }
-	  else if (schnang) {
-	    v_gtext(sn,x,y,w,textarr[TX_SCHN_ANGE]);
-	  }
-	  else if (handsp) {
-	    v_gtext(sn,x,y,w,textarr[TX_HAND]);
+	  else {
+	    v_gtext(sn,x,y,0,textarr[TX_OUVE]);
 	  }
+	}
+	else if (schwang) {
+	  v_gtext(sn,x,y,0,textarr[TX_SCHW_ANGE]);
+	}
+	else if (schnang) {
+	  v_gtext(sn,x,y,0,textarr[TX_SCHN_ANGE]);
+	}
+	else if (handsp && !revolang) {
+	  v_gtext(sn,x,y,0,textarr[TX_HAND]);
+	}
       }
       else {
 	if (th>=0) {
-	  v_gtext(sn,x,y,w,clr);
+	  v_gtext(sn,x,y,0,clr);
 	  if (s==th) {
-	    v_gtext(sn,x,y,w,textarr[TX_UEBERLEGT]);
+	    v_gtext(sn,x,y,0,textarr[TX_UEBERLEGT]);
 	  }
-	  v_gtext(sn,x,y+charh[sn],w,clr);
+	  v_gtext(sn,x,y+charh[sn],0,clr);
 	}
       }
     }
@@ -877,7 +962,7 @@ VOID di_hand()
 VOID next_grandhand(sn)
 int sn;
 {
-  sn=(sn+1)%3;
+  sn=left(sn);
   if (sn==hoerer) {
     ktrply=-1;
     init_ramsch();
@@ -896,7 +981,6 @@ int sn;
   }
   else {
     ktrply=sn;
-    if (actdial[sn]) remove_di(sn);
     create_di(sn,digrandhand);
   }
 }
@@ -907,7 +991,6 @@ int sn,s;
   static char txt[20];
 
   if (lost[sn]) return;
-  if (actdial[sn]) remove_di(sn);
   diterm[sn][2].str=txt;
   prspnam(txt,s);
   create_di(sn,diterm[sn]);
@@ -927,51 +1010,46 @@ int sn;
 
 VOID di_ansage()
 {
-  static char txt1[30],txt2[30];
+  static char txt[30];
   int i,ktr;
 
   ktrply=-1;
-  diansage[2].str=txt1;
-  dikontra[2].str=txt2;
-  dikonre[2].str=txt2;
-  strcpy(txt1,textarr[TX_NULL+trumpf+1]);
-  strcpy(txt2,txt1);
-  if (handsp) {
-    strcat(txt1," ");
-    strcat(txt1,textarr[TX_HAND]);
-  }
+  diansage[2].str=dikontra[2].str=dikonre[2].str=txt;
+  strcpy(txt,textarr[revolang?TX_REVOLUTION:TX_NULL+trumpf+1]);
   if (trumpf==-1) {
-    if (ouveang) {
-      strcat(txt2," ");
-      strcat(txt2,textarr[TX_OUVE]);
-    }
-    if (handsp) {
-      strcat(txt2," ");
-      strcat(txt2,textarr[TX_HAND]);
+    if (!revolang) {
+      if (ouveang) {
+	strcat(txt," ");
+	strcat(txt,textarr[TX_OUVE]);
+      }
+      if (handsp) {
+	strcat(txt," ");
+	strcat(txt,textarr[TX_HAND]);
+      }
     }
   }
   else {
     if (ouveang) {
-      strcat(txt2," ");
-      strcat(txt2,textarr[TX_OUVE]);
+      strcat(txt," ");
+      strcat(txt,textarr[TX_OUVE]);
     }
     else if (schwang) {
-      strcat(txt2," ");
-      strcat(txt2,textarr[TX_SCHWARZ]);
+      strcat(txt," ");
+      strcat(txt,textarr[TX_SCHWARZ]);
     }
     else if (schnang) {
-      strcat(txt2," ");
-      strcat(txt2,textarr[TX_SCHNEIDER]);
+      strcat(txt," ");
+      strcat(txt,textarr[TX_SCHNEIDER]);
     }
     else if (handsp) {
-      strcat(txt2," ");
-      strcat(txt2,textarr[TX_HAND]);
+      strcat(txt," ");
+      strcat(txt,textarr[TX_HAND]);
     }
   }
   ktr=0;
   ktrnext=-1;
   if (playkontra) {
-    for (i=0;i<numsp;i++) {
+    for (i=0;i<3;i++) {
       if (i!=spieler && (playkontra==1 || sagte18[i])) {
 	if (!ktr) {
 	  ktr=1;
@@ -988,11 +1066,10 @@ VOID di_ansage()
       }
     }
   }
-  if (numsp==1 && iscomp(spieler)) {
-    if (ktr) di_kontra(0);
-    else create_di(0,diansage);
+  if (numsp==1 && iscomp(spieler) && !ktr) {
+    create_di(0,diansage);
   }
-  else if (numsp>=2 && ktr) {
+  else if (ktr) {
     di_kontra(ktrsag);
   }
   else do_angesagt();
@@ -1003,11 +1080,15 @@ int sn;
 {
   ktrply=sn;
   ktrsag=sn;
-  if (actdial[sn]) remove_di(sn);
   sort2[sn]=sort2[spieler]=trumpf==-1;
   if (!iscomp(spieler)) initscr(spieler);
-  initscr(sn);
-  create_di(sn,dikontra);
+  if (iscomp(sn)) {
+    di_ktrnext(sn,sage_kontra(sn));
+  }
+  else {
+    initscr(sn);
+    create_di(sn,dikontra);
+  }
 }
 
 VOID di_rekontra(sn)
@@ -1019,8 +1100,12 @@ int sn;
   direkontra[2].str=txt;
   strcpy(txt,textarr[TX_VON]);
   prspnam(txt+strlen(txt),sn);
-  if (actdial[spieler]) remove_di(spieler);
-  create_di(spieler,direkontra);
+  if (iscomp(spieler)) {
+    di_ktrnext(sn,sage_re(spieler));
+  }
+  else {
+    create_di(spieler,direkontra);
+  }
 }
 
 VOID di_konre(sn)
@@ -1035,10 +1120,143 @@ int sn;
     strcat(txt," & ");
     strcat(txt,textarr[TX_RE]);
   }
-  if (actdial[sn]) remove_di(sn);
   create_di(sn,dikonre);
 }
 
+VOID di_ktrnext(sn,f)
+int sn,f;
+{
+  if (kontrastufe==1) {
+    if (f) {
+      kontrastufe=2;
+      prot2.verdopp[spieler]=2;
+      ktrnext=left(ktrsag)==spieler?right(ktrsag):left(ktrsag);
+      if (iscomp(ktrnext)) ktrnext=-1;
+      if (iscomp(ktrsag)) ktrsag=ktrnext,ktrnext=-1;
+    }
+    else {
+      ktrnext=-1;
+      ktrsag=left(ktrsag)==spieler?right(ktrsag):left(ktrsag);
+      if (iscomp(ktrsag)) ktrsag=ktrnext;
+    }
+    if (ktrsag>=0) {
+      di_konre(ktrsag);
+    }
+    else {
+      do_angesagt();
+    }
+  }
+  else if (f) {
+    kontrastufe=1;
+    prot2.verdopp[sn]=2;
+    di_rekontra(sn);
+  }
+  else if (ktrnext>=0 && ktrnext!=sn) {
+    di_kontra(ktrnext);
+    ktrnext=-1;
+  }
+  else {
+    if (numsp==1 && iscomp(spieler) && playkontra==2 && !sagte18[0]) {
+      create_di(0,diansage);
+    }
+    else do_angesagt();
+  }
+}
+
+VOID di_dicht()
+{
+  phase=NULLDICHT;
+  dichtply=spieler;
+  if (iscomp(spieler)) {
+    nullendscr();
+  }
+  else {
+    create_di(spieler,didicht);
+  }
+}
+
+VOID di_weiter(ini)
+int ini;
+{
+  int x,y;
+
+  if (!ini || iscomp(dichtply)) dichtply=left(dichtply);
+  while (iscomp(dichtply) && dichtply!=spieler) dichtply=left(dichtply);
+  if ((!ini && dichtply==spieler)
+      || !numsp) {
+    finishgame();
+    return;
+  }
+  x=(desk[dichtply].w-diweiter[0].w*charw[dichtply])/2;
+  y=(desk[dichtply].playy+desk[dichtply].skaty+desk[dichtply].cardh+
+     -diweiter[0].h*charh[dichtply])/2;
+  if (actdial[dichtply]) remove_di(dichtply);
+  create_dial(dichtply,x,y,0,diweiter);
+  actdial[dichtply]=diweiter;
+}
+
+VOID di_klopfen(sn)
+int sn;
+{
+  create_di(sn,diklopfen);
+}
+
+VOID di_schenken(sn)
+int sn;
+{
+  int msp;
+
+  if (schenkstufe) return;
+  phase=SCHENKEN;
+  schenkstufe=1;
+  schenknext=sn;
+  msp=left(sn)==spieler?left(spieler):left(sn);
+  if (iscomp(msp)) {
+    if (kontrastufe) di_nichtschenken(msp);
+    else di_geschenkt();
+  }
+  else {
+    create_di(msp,dischenken);
+    schenkply=msp;
+  }
+}
+
+VOID di_geschenkt()
+{
+  if (iscomp(spieler)) {
+    finishgame();
+  }
+  else {
+    digeschenkt[4].str=textarr[schnang?TX_SCHWARZ:TX_SCHNEIDER];
+    create_di(spieler,schwang || trumpf==-1?diendeschenken:digeschenkt);
+    schenkply=spieler;
+  }
+}
+
+VOID di_wiederschenken(sn,f)
+int sn,f;
+{
+  int msp;
+
+  msp=left(sn)==spieler?left(spieler):left(sn);
+  schenknext=f && !iscomp(msp)?msp:-1;
+  create_di(sn,diwiederschenken);
+  schenkply=sn;
+}
+
+VOID di_nichtschenken(sn)
+int sn;
+{
+  sn=left(sn)==spieler?left(spieler):left(sn);
+  if (!iscomp(sn)) {
+    create_di(sn,dinichtschenken);
+    schenkply=sn;
+  }
+  else {
+    phase=SPIELEN;
+  }
+}
+
 VOID di_schieben()
 {
   int sn;
@@ -1051,25 +1269,25 @@ VOID di_schieben()
       if (comp_sramsch(sn)) {
 	return;
       }
-      vmh=(vmh+1)%3;
+      vmh=left(vmh);
     }
     else {
-      if (actdial[sn]) remove_di(sn);
-      create_di(sn,dischieben);
+      create_di(sn,playsramsch?dischieben:diklopfen);
       return;
     }
   } while (vmh);
   start_ramsch();
 }
 
-int di_verdoppelt(f)
-int f;
+int di_verdoppelt(f,kl)
+int f,kl;
 {
   int sn;
   static int verd1,verd2;
   static char txt[20];
 
   if (!f) {
+    diverdoppelt[3].str=textarr[kl?TX_KLOPFT:TX_SCHIEBT];
     prspnam(txt,spieler);
     diverdoppelt[2].str=txt;
     verd1=verd2=-1;
@@ -1080,18 +1298,17 @@ int f;
       }
     }
     sramschstufe++;
-    prot2.verdopp[spieler]=1;
+    prot2.verdopp[spieler]=kl+1;
   }
   if (verd1!=-1) {
     spieler=verd1;
-    if (actdial[spieler]) remove_di(spieler);
     create_di(spieler,diverdoppelt);
     verd1=verd2;
     verd2=-1;
     return 1;
   }
   if (!iscomp(spieler)) {
-    vmh=(vmh+1)%3;
+    vmh=left(vmh);
     if (vmh) di_schieben();
     else start_ramsch();
   }
@@ -1109,7 +1326,10 @@ VOID di_spiel()
 
   ktrply=-1;
   sprintf(txt,textarr[TX_GEREIZT_BIS_N],reizw[reizp]);
-  dispiel[12].str=txt;
+  dispiel[14].str=txt;
+  dispiel[11].spec=spitzezaehlt && kannspitze?OB_BUTTON:OB_HIDDEN;
+  dispiel[13].spec=revolution?OB_BUTTON:OB_HIDDEN;
+  dispiel[7].next=revolution?6:-5;
   create_di(spieler,dispiel);
 }
 
@@ -1213,12 +1433,10 @@ FILE *f;
   trumpf=prot1.trumpf;
   for (s=0;s<3;s++) {
     for (i=0;i<10;i++) stiche[i][s]=prot1.stiche[i][s];
-    if (protsort[sn]) {
-      for (i=0;i<9;i++) {
-        for (j=i+1;j<10;j++) {
-          if (lower(stiche[i][s],stiche[j][s],trumpf==-1)) {
-            swap(&stiche[i][s],&stiche[j][s]);
-	  }
+    for (i=(protsort[sn]?0:prot1.stichgem);i<9;i++) {
+      for (j=i+1;j<10;j++) {
+	if (lower(stiche[i][s],stiche[j][s],trumpf==-1)) {
+	  swap(&stiche[i][s],&stiche[j][s]);
         }
       }
     }
@@ -1233,7 +1451,7 @@ FILE *f;
       }
       else {
 	if (i && prot1.stichgem<=i) {
-	  e=OB_HIDDEN;
+	  e=prot1.gewonn && prot1.stichgem?OB_NONE:OB_HIDDEN;
 	}
 	else {
 	  e=prot1.anspiel[i]==s?OB_UNDERLINED:OB_NONE;
@@ -1243,11 +1461,22 @@ FILE *f;
       diproto[sn][8+3*i+s].spec=e;
       diproto[sn][8+3*i+s].str=txt[sn][i][s];
       if (prot1.stichgem || protsort[sn]) {
-	strcpy(txt[sn][i][s],textarr[TX_KARO+(stiche[i][s]>>3)]);
-	strcat(txt[sn][i][s],textarr[TX_A+(stiche[i][s]&7)]);
+	if (prot1.spitze &&
+	    stiche[i][s]==(prot1.trumpf==4?BUBE:SIEBEN|prot1.trumpf<<3)) {
+	  strcpy(txt[sn][i][s],textarr[TX_SPITZE_PROT]);
+	}
+	else {
+	  strcpy(txt[sn][i][s],textarr[TX_KARO+(stiche[i][s]>>3)]);
+	  strcat(txt[sn][i][s],textarr[TX_A+(stiche[i][s]&7)]);
+	}
       }
       else {
-	strcpy(txt[sn][i][s],textarr[TX_PASSE]);
+	if (e==OB_UNDERLINED) strcpy(txt[sn][i][s]," ");
+	else txt[sn][i][s][0]=0;
+	strcat(txt[sn][i][s],
+	       textarr[prot1.schenken?
+		       prot1.spieler==s?TX_AKZEPTIERT:TX_SCHENKEN:TX_PASSE]);
+	if (e==OB_UNDERLINED) strcat(txt[sn][i][s]," ");
       }
     }
     if (f && diproto[sn][8+3*i].spec!=OB_HIDDEN) {
@@ -1278,7 +1507,7 @@ int sn,ini,log;
   static char vhschob[3][40],mhschob[3][40];
   char *hd="----------------------------------------\n";
   char *tl="========================================\n";
-  char hdbuf[100];
+  char hdbuf[100],spn[12];
   int s,p,u1,u2,u3;
   FILE *f;
 
@@ -1290,17 +1519,43 @@ int sn,ini,log;
       return;
     }
     for (s=0;s<3;s++) {
-      pformat(f,OB_CENTERED,spnames[s][0],1);
+      if (prot1.verdopp[s]==2) {
+	strcpy(spn," ");
+	strcat(spn,spnames[s][0]);
+	strcat(spn," ");
+	p=OB_CENTERED|OB_UNDERLINED;
+      }
+      else {
+	strcpy(spn,spnames[s][0]);
+	p=OB_CENTERED;
+      }
+      pformat(f,p,spn,1);
     }
     fputc('\n',f);
     for (s=0;s<3;s++) {
-      pformat(f,OB_CENTERED,spnames[s][1],1);
+      if (prot1.verdopp[s]==2 && spnames[s][1][0]) {
+	strcpy(spn," ");
+	strcat(spn,spnames[s][1]);
+	strcat(spn," ");
+	p=OB_CENTERED|OB_UNDERLINED;
+      }
+      else {
+	strcpy(spn,spnames[s][1]);
+	p=OB_CENTERED;
+      }
+      pformat(f,p,spn,1);
     }
     fprintf(f,"\n%s",hd);
   }
   else f=0;
   if (ini) protsort[sn]=0;
   set_names(diproto[sn],2);
+  for (p=0;p<2;p++) {
+    for (s=0;s<3;s++) {
+      diproto[sn][2+p*3+s].spec=prot1.verdopp[s]==2?
+	OB_CENTERED|OB_UNDERLINED:OB_CENTERED;
+    }
+  }
   diproto[sn][44].str=txt[sn];
   if (prot1.trumpf==5 && prot1.augen==0 && !prot1.gewonn) {
     strcpy(txt[sn],textarr[TX_NIEMAND]);
@@ -1314,7 +1569,8 @@ int sn,ini,log;
   }
   else {
     strcat(txt[sn],textarr[TX_SPIELTE]);
-    strcat(txt[sn],textarr[TX_NULL+prot1.trumpf+1]);
+    strcat(txt[sn],textarr[prot1.revolution?TX_REVOLUTION:
+			   TX_NULL+prot1.trumpf+1]);
   }
   diproto[sn][38].str=textarr[TX_IM_SKAT_IST];
   diproto[sn][38].spec=OB_NONE;
@@ -1335,12 +1591,12 @@ int sn,ini,log;
   sprintf(bis[sn],"%d",prot1.gereizt);
   if (prot1.sramsch) {
     p=protsort[sn];
-    u1=prot1.verdopp[prot1.anspiel[0]]?OB_UNDERLINED:OB_NONE;
-    u2=prot1.verdopp[(prot1.anspiel[0]+1)%3]?OB_UNDERLINED:OB_NONE;
-    u3=prot1.verdopp[(prot1.anspiel[0]+2)%3]?OB_UNDERLINED:OB_NONE;
+    u1=prot1.verdopp[prot1.anspiel[0]]==1?OB_UNDERLINED:OB_NONE;
+    u2=prot1.verdopp[left(prot1.anspiel[0])]==1?OB_UNDERLINED:OB_NONE;
+    u3=prot1.verdopp[right(prot1.anspiel[0])]==1?OB_UNDERLINED:OB_NONE;
   }
   else {
-    p=protsort[sn] && prot1.trumpf==-1 && prot1.ehsso==4?0:prot1.ehsso;
+    p=protsort[sn] && !prot1.handsp?0:prot1.ehsso;
   }
   if (p) {
     if (prot1.sramsch) {
@@ -1384,13 +1640,19 @@ int sn,ini,log;
     }
   }
   if (!prot1.stichgem) {
-    diproto[sn][42].spec=OB_HIDDEN;
-    diproto[sn][40].str=OB_NONE;
-    diproto[sn][41].str=OB_NONE;
-    diproto[sn][43].str=OB_NONE;
-    diproto[sn][44].str=OB_NONE;
-    diproto[sn][45].str=OB_NONE;
-    diproto[sn][46].str=OB_NONE;
+    if (prot1.schenken) {
+      diproto[sn][45].str=textarr[TX_GEWONNEN];
+      diproto[sn][46].str=OB_NONE;
+    }
+    else {
+      diproto[sn][42].spec=OB_HIDDEN;
+      diproto[sn][40].str=OB_NONE;
+      diproto[sn][41].str=OB_NONE;
+      diproto[sn][43].str=OB_NONE;
+      diproto[sn][44].str=OB_NONE;
+      diproto[sn][45].str=OB_NONE;
+      diproto[sn][46].str=OB_NONE;
+    }
   }
   else if (prot1.trumpf==-1) {
     diproto[sn][45].str=textarr[prot1.gewonn?TX_GEWONNEN:TX_VERLOREN];
@@ -1418,7 +1680,7 @@ int sn,ini,log;
     prot_fun(sn,f);
     fputs(hd,f);
     fprintf(f,"%s %s\n",textarr[TX_IM_SKAT_IST],imski[sn]);
-    if (prot1.stichgem) {
+    if (prot1.stichgem || prot1.schenken) {
       if (prot1.trumpf!=5) {
 	fprintf(f,"%s %s\n",textarr[TX_IM_SKAT_WAR],imskw[sn]);
 	if (prot1.gereizt) {
@@ -1440,7 +1702,9 @@ int sn,ini,log;
       }
       if (prot1.trumpf!=5 || prot1.augen!=0 || prot1.gewonn) {
 	fprintf(f,"%s",diproto[sn][45].str);
-	if (diproto[sn][46].str) fprintf(f," %s",diproto[sn][46].str);
+	if (diproto[sn][46].str!=OB_NONE) {
+	  fprintf(f," %s",diproto[sn][46].str);
+	}
 	fputc('\n',f);
       }
     }
@@ -1483,6 +1747,7 @@ int be;
      (mes1?textarr[TX_UEBERREIZT]:
       mes2?textarr[TX_SCHNEIDERFREI]:
       mes3?textarr[TX_NICHT_SCHWARZ]:
+      mes4?textarr[TX_SPITZE_VERLOREN]:
       OB_NONE));
   diresult[3].str=sg;
   if (trumpf==5 && spwert==0) {
@@ -1501,7 +1766,10 @@ int be;
     strcpy(sa,textarr[TX_DEN_RAMSCH]);
   }
   else {
-    if (schwz || !nullv) {
+    if (stich==1) {
+      *sa=0;
+    }
+    else if ((spgew && schwz) || !nullv) {
       strcpy(sa,textarr[TX_DAS_SPIEL_SCHWARZ]);
     }
     else {
@@ -1598,6 +1866,14 @@ int sn;
   sprintf(txt[sn],"%s %d",textarr[TX_BOCK_SPIELE],bockspiele);
   dioptions[sn][18].spec=splstp||splres?OB_EXIT:OB_HIDDEN;
   dioptions[sn][18].str=textarr[splres?TX_PROTOKOLL:TX_SPIELLISTE];
+  dioptions[sn][17].spec=
+    schenken && !schenkstufe && trumpf<=4 &&
+      stich==1 && phase==SPIELEN && sn!=spieler
+	?OB_EXIT:OB_HIDDEN;
+  dioptions[sn][14].spec=
+    dioptions[sn][15].spec=
+      dioptions[sn][16].spec=
+	dioptions[sn][17].spec==OB_EXIT?OB_HIDDEN:OB_NONE;
   create_di(sn,dioptions[sn]);
   dioptions[sn][5-sort1[sn]].spec|=OB_SELECTED;
   dioptions[sn][7-alternate[sn]].spec|=OB_SELECTED;
@@ -1627,6 +1903,10 @@ int sn;
   divarianten[sn][10+playkontra].spec|=OB_SELECTED;
   divarianten[sn][14+playbock].spec|=OB_SELECTED;
   divarianten[sn][18+resumebock].spec|=OB_SELECTED;
+  divarianten[sn][21+spitzezaehlt].spec|=OB_SELECTED;
+  divarianten[sn][25+revolution].spec|=OB_SELECTED;
+  divarianten[sn][28+klopfen].spec|=OB_SELECTED;
+  divarianten[sn][31+schenken].spec|=OB_SELECTED;
 }
 
 VOID di_bockevents(sn)
@@ -1655,8 +1935,9 @@ int sn;
   prverz(sn);
   for (i=0;i<3;i++) {
     diverzoegerung[sn][4+i].str=kl+i;
-    diverzoegerung[sn][8+i].str=gr+2-i;
+    diverzoegerung[sn][7+i].str=gr+2-i;
   }
+  diverzoegerung[sn][12-fastdeal].spec|=OB_SELECTED;
 }
 
 VOID di_menubutton(sn)
