@@ -39,7 +39,7 @@ typedef struct {
 
 EXTERN OBJECT *actdial[3];
 EXTERN OBJECT *resdial[3];
-EXTERN int ktrsag,ktrnext;
+EXTERN int ktrsag,ktrnext,schenknext;
 
 EXTERN OBJECT dihand[]
 #ifdef XDIAL_C
@@ -102,6 +102,83 @@ EXTERN OBJECT dibuben[]
 #endif
 ;
 
+EXTERN OBJECT diklopfen[]
+#ifdef XDIAL_C
+=
+{
+  {5,0,0,27,7,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,25,1,TOB(TX_KLOPFEN_T),OB_NONE},
+  {OB_CENTERED,1,3,25,1,TOB(TX_SPIELWERT_VERDOPPELN),OB_NONE},
+  {OB_EXIT,6,5,6,1,TOB(TX_NEIN),OB_NONE},
+  {OB_EXIT,15,5,6,1,TOB(TX_JA),OB_NONE}
+}
+#endif
+;
+
+EXTERN OBJECT dischenken[]
+#ifdef XDIAL_C
+=
+{
+  {5,0,0,27,7,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,25,1,TOB(TX_SCHENKEN_T),OB_NONE},
+  {OB_CENTERED,1,3,25,1,TOB(TX_SPIEL_VERLOREN_GEBEN),OB_NONE},
+  {OB_EXIT,6,5,6,1,TOB(TX_NEIN),OB_NONE},
+  {OB_EXIT,15,5,6,1,TOB(TX_JA),OB_NONE}
+}
+#endif
+;
+
+EXTERN OBJECT digeschenkt[]
+#ifdef XDIAL_C
+=
+{
+  {5,0,0,27,7,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,25,1,TOB(TX_SCHENKEN_T),OB_NONE},
+  {OB_CENTERED,1,3,25,1,TOB(TX_GESCHENK_ANNEHMEN),OB_NONE},
+  {OB_EXIT,2,5,11,1,TOB(TX_JA),OB_NONE},
+  {OB_EXIT,14,5,11,1,TOB(TX_SCHNEIDER),OB_NONE}
+}
+#endif
+;
+
+EXTERN OBJECT diendeschenken[]
+#ifdef XDIAL_C
+=
+{
+  {4,0,0,27,7,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,25,1,TOB(TX_SCHENKEN_T),OB_NONE},
+  {OB_CENTERED,1,3,25,1,TOB(TX_GEGNER_GEBEN_AUF),OB_NONE},
+  {OB_EXIT,10,5,7,1,TOB(TX_AHA),OB_NONE}
+}
+#endif
+;
+
+EXTERN OBJECT diwiederschenken[]
+#ifdef XDIAL_C
+=
+{
+  {6,0,0,27,9,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,25,1,TOB(TX_SCHENKEN_T),OB_NONE},
+  {OB_CENTERED,1,3,25,1,TOB(TX_SCHN_ANGE),OB_NONE},
+  {OB_CENTERED,1,5,25,1,TOB(TX_SPIEL_VERLOREN_GEBEN),OB_NONE},
+  {OB_EXIT,6,7,6,1,TOB(TX_NEIN),OB_NONE},
+  {OB_EXIT,15,7,6,1,TOB(TX_JA),OB_NONE}
+}
+#endif
+;
+
+EXTERN OBJECT dinichtschenken[]
+#ifdef XDIAL_C
+=
+{
+  {4,0,0,27,7,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,25,1,TOB(TX_SCHENKEN_T),OB_NONE},
+  {OB_CENTERED,1,3,25,1,TOB(TX_MITSPIELER_LEHNT_AB),OB_NONE},
+  {OB_EXIT,8,5,11,1,TOB(TX_AHA),OB_NONE}
+}
+#endif
+;
+
 EXTERN OBJECT diende[3][4]
 #ifdef XDIAL_C
 =
@@ -143,7 +220,7 @@ EXTERN OBJECT dispiel[]
 #ifdef XDIAL_C
 =
 {
-  {13,0,0,43,13,OB_NONE,OB_NONE},
+  {15,0,0,43,13,OB_NONE,OB_NONE},
   {OB_CENTERED|OB_UNDERLINED,1,1,41,1,TOB(TX_SPIEL),OB_NONE},
   {OB_BUTTON,2,3,9,1,TOB(TX_KARO),1},
   {OB_BUTTON,12,3,9,1,TOB(TX_HERZ),1},
@@ -154,7 +231,9 @@ EXTERN OBJECT dispiel[]
   {OB_BUTTON,2,7,12,1,TOB(TX_SCHNEIDER),OB_NONE},
   {OB_BUTTON,15,7,13,1,TOB(TX_SCHWARZ),OB_NONE},
   {OB_BUTTON,29,7,12,1,TOB(TX_OUVERT),OB_NONE},
+  {OB_BUTTON,2,9,12,1,TOB(TX_SPITZE),OB_NONE},
   {OB_EXIT,15,9,13,1,TOB(TX_ANSAGEN),OB_NONE},
+  {OB_BUTTON,29,9,12,1,TOB(TX_REVOLUTION),-11},
   {OB_CENTERED,1,11,41,1,OB_NONE,OB_NONE}
 }
 #endif
@@ -179,13 +258,27 @@ EXTERN OBJECT diueberr[]
 #ifdef XDIAL_C
 =
 {
-  {7,0,0,43,11,OB_NONE,OB_NONE},
+  {8,0,0,43,12,OB_NONE,OB_NONE},
   {OB_CENTERED|OB_UNDERLINED,1,1,41,1,TOB(TX_HINWEIS),OB_NONE},
   {OB_CENTERED,1,3,41,1,TOB(TX_U1),OB_NONE},
   {OB_CENTERED,1,4,41,1,TOB(TX_U2),OB_NONE},
   {OB_CENTERED,1,6,41,1,TOB(TX_U3),OB_NONE},
   {OB_CENTERED,1,7,41,1,TOB(TX_U4),OB_NONE},
-  {OB_EXIT,17,9,9,1,TOB(TX_AHA),OB_NONE}
+  {OB_CENTERED,1,8,41,1,TOB(TX_U5),OB_NONE},
+  {OB_EXIT,17,10,9,1,TOB(TX_AHA),OB_NONE}
+}
+#endif
+;
+
+EXTERN OBJECT dispitze[]
+#ifdef XDIAL_C
+=
+{
+  {5,0,0,43,8,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,41,1,TOB(TX_HINWEIS),OB_NONE},
+  {OB_CENTERED,1,3,41,1,TOB(TX_SPITZE_F1),OB_NONE},
+  {OB_CENTERED,1,4,41,1,TOB(TX_SPITZE_F2),OB_NONE},
+  {OB_EXIT,17,6,9,1,TOB(TX_AHA),OB_NONE}
 }
 #endif
 ;
@@ -194,9 +287,9 @@ EXTERN OBJECT diansage[]
 #ifdef XDIAL_C
 =
 {
-  {3,0,0,17,6,OB_NONE,OB_NONE},
-  {OB_CENTERED|OB_UNDERLINED,1,1,15,1,TOB(TX_ANGESAGT_IST),OB_NONE},
-  {OB_EXIT,3,4,11,1,OB_NONE,OB_NONE}
+  {3,0,0,19,5,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,17,1,TOB(TX_ANGESAGT_IST),OB_NONE},
+  {OB_EXIT,1,3,17,1,OB_NONE,OB_NONE}
 }
 #endif
 ;
@@ -240,6 +333,29 @@ EXTERN OBJECT dikonre[]
 #endif
 ;
 
+EXTERN OBJECT didicht[]
+#ifdef XDIAL_C
+=
+{
+  {5,0,0,24,7,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,22,1,TOB(TX_NULL_DICHT_T),OB_NONE},
+  {OB_CENTERED,1,3,22,1,TOB(TX_KARTEN_AUFDECKEN),OB_NONE},
+  {OB_EXIT,5,5,6,1,TOB(TX_JA),OB_NONE},
+  {OB_EXIT,13,5,6,1,TOB(TX_NEIN),OB_NONE}
+}
+#endif
+;
+
+EXTERN OBJECT diweiter[]
+#ifdef XDIAL_C
+=
+{
+  {2,0,0,14,3,OB_NONE,OB_NONE},
+  {OB_EXIT,2,1,10,1,TOB(TX_WEITER),OB_NONE}
+}
+#endif
+;
+
 EXTERN OBJECT diwieder[]
 #ifdef XDIAL_C
 =
@@ -314,8 +430,8 @@ EXTERN OBJECT diproto[3][50]
 #ifdef XDIAL_C
 =
 {{
-  {50,0,0,40,23,OB_NONE,OB_NONE},
-  {OB_CENTERED|OB_UNDERLINED,1,1,38,1,TOB(TX_PROTOKOLL_T),OB_NONE},
+  {50,0,0,41,23,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,39,1,TOB(TX_PROTOKOLL_T),OB_NONE},
   {OB_CENTERED,2,3,12,1,OB_NONE,OB_NONE},
   {OB_CENTERED,14,3,12,1,OB_NONE,OB_NONE},
   {OB_CENTERED,26,3,12,1,OB_NONE,OB_NONE},
@@ -354,16 +470,16 @@ EXTERN OBJECT diproto[3][50]
   {OB_NONE,28,14,11,1,OB_NONE,OB_NONE},
   {OB_NONE,2,15,13,1,OB_NONE,OB_NONE},
   {OB_NONE,16,15,22,1,OB_NONE,OB_NONE},
-  {OB_NONE,2,16,37,1,OB_NONE,OB_NONE},
+  {OB_NONE,2,16,38,1,OB_NONE,OB_NONE},
   {OB_NONE,16,16,22,1,OB_NONE,OB_NONE},
   {OB_NONE,2,17,13,1,OB_NONE,OB_NONE},
   {OB_NONE,16,17,22,1,OB_NONE,OB_NONE},
-  {OB_NONE,2,18,37,1,OB_NONE,OB_NONE},
+  {OB_NONE,2,18,38,1,OB_NONE,OB_NONE},
   {OB_NONE,2,19,13,1,OB_NONE,OB_NONE},
   {OB_NONE,16,19,23,1,OB_NONE,OB_NONE},
   {OB_EXIT,2,21,12,1,TOB(TX_SPIELLISTE),OB_NONE},
   {OB_EXIT,17,21,12,1,TOB(TX_WEITER),OB_NONE},
-  {OB_EXIT,32,21,6,1,OB_NONE,OB_NONE}
+  {OB_EXIT,32,21,7,1,OB_NONE,OB_NONE}
 }}
 #endif
 ;
@@ -456,10 +572,10 @@ EXTERN OBJECT dioptions[3][22]
   {OB_NONE,2,10,35,1,TOB(TX_GEREIZT_BIS_L),OB_NONE},
   {OB_NONE,2,11,35,1,TOB(TX_GESPIELT_WIRD),OB_NONE},
   {OB_NONE,2,12,35,1,TOB(TX_LETZTER_STICH),OB_NONE},
-  {OB_NONE,2,12,35,1,TOB(TX_GEDRUECKT),OB_NONE},
   {OB_NONE,5,13,32,1,OB_NONE,OB_NONE},
   {OB_NONE,5,14,32,1,OB_NONE,OB_NONE},
   {OB_NONE,5,15,32,1,OB_NONE,OB_NONE},
+  {OB_EXIT,13,15,12,1,TOB(TX_SCHENKEN),OB_NONE},
   {OB_EXIT,5,17,13,1,TOB(TX_SPIELLISTE),OB_NONE},
   {OB_EXIT,20,17,13,1,TOB(TX_OPTIONEN),OB_NONE},
   {OB_EXIT,5,19,13,1,TOB(TX_ENDE),OB_NONE},
@@ -468,21 +584,19 @@ EXTERN OBJECT dioptions[3][22]
 #endif
 ;
 
-EXTERN OBJECT dicopyr[3][11]
+EXTERN OBJECT dicopyr[3][9]
 #ifdef XDIAL_C
 =
 {{
-  {11,0,0,40,16,OB_NONE,OB_NONE},
+  {9,0,0,40,13,OB_NONE,OB_NONE},
   {OB_CENTERED|OB_UNDERLINED,1,1,38,1,TOB(TX_XSKAT),OB_NONE},
   {OB_CENTERED,1,3,38,1,TOB(TX_COPYRIGHT),OB_NONE},
   {OB_CENTERED,1,4,38,1,TOB(TX_EMAIL),OB_NONE},
-  {OB_CENTERED,1,6,38,1,TOB(TX_HOMEPAGE),OB_NONE},
-  {OB_CENTERED,1,7,38,1,TOB(TX_URL),OB_NONE},
-  {OB_CENTERED,1,9,38,1,TOB(TX_FR1),OB_NONE},
-  {OB_CENTERED,1,10,38,1,TOB(TX_FR2),OB_NONE},
-  {OB_CENTERED,1,11,38,1,TOB(TX_FR3),OB_NONE},
-  {OB_CENTERED,1,12,38,1,TOB(TX_FR4),OB_NONE},
-  {OB_EXIT,13,14,14,1,TOB(TX_WEITER),OB_NONE}
+  {OB_CENTERED,1,6,38,1,TOB(TX_FR1),OB_NONE},
+  {OB_CENTERED,1,7,38,1,TOB(TX_FR2),OB_NONE},
+  {OB_CENTERED,1,8,38,1,TOB(TX_FR3),OB_NONE},
+  {OB_CENTERED,1,9,38,1,TOB(TX_FR4),OB_NONE},
+  {OB_EXIT,13,11,14,1,TOB(TX_WEITER),OB_NONE}
 }}
 #endif
 ;
@@ -520,11 +634,11 @@ EXTERN OBJECT distrateg[3][25]
 #endif
 ;
 
-EXTERN OBJECT divarianten[3][21]
+EXTERN OBJECT divarianten[3][34]
 #ifdef XDIAL_C
 =
 {{
-  {21,0,0,42,11,OB_NONE,OB_NONE},
+  {34,0,0,42,15,OB_NONE,OB_NONE},
   {OB_CENTERED|OB_UNDERLINED,1,1,40,1,TOB(TX_VARIANTEN),OB_NONE},
   {OB_NONE,1,3,20,1,TOB(TX_RAMSCH_SPIELEN),OB_NONE},
   {OB_BUTTON,17,3,7,1,TOB(TX_NEIN),1},
@@ -544,7 +658,20 @@ EXTERN OBJECT divarianten[3][21]
   {OB_NONE,3,7,18,1,TOB(TX_FORTSETZEN),OB_NONE},
   {OB_BUTTON,17,7,7,1,TOB(TX_NEIN),1},
   {OB_BUTTON,24,7,7,1,TOB(TX_JA),-1},
-  {OB_EXIT,14,9,14,1,TOB(TX_WEITER),OB_NONE}
+  {OB_NONE,1,8,18,1,TOB(TX_SPITZE),OB_NONE},
+  {OB_BUTTON,17,8,7,1,TOB(TX_NEIN),1},
+  {OB_BUTTON,24,8,7,1,TOB(TX_JA),1},
+  {OB_BUTTON,31,8,10,1,TOB(TX_PLUS2),-2},
+  {OB_NONE,1,9,18,1,TOB(TX_REVOLUTION),OB_NONE},
+  {OB_BUTTON,17,9,7,1,TOB(TX_NEIN),1},
+  {OB_BUTTON,24,9,7,1,TOB(TX_JA),-1},
+  {OB_NONE,1,10,18,1,TOB(TX_KLOPFEN),OB_NONE},
+  {OB_BUTTON,17,10,7,1,TOB(TX_NEIN),1},
+  {OB_BUTTON,24,10,7,1,TOB(TX_JA),-1},
+  {OB_NONE,1,11,18,1,TOB(TX_SCHENKEN),OB_NONE},
+  {OB_BUTTON,17,11,7,1,TOB(TX_NEIN),1},
+  {OB_BUTTON,24,11,7,1,TOB(TX_JA),-1},
+  {OB_EXIT,14,13,14,1,TOB(TX_WEITER),OB_NONE}
 }}
 #endif
 ;
@@ -568,21 +695,24 @@ EXTERN OBJECT dibockevents[3][11]
 #endif
 ;
 
-EXTERN OBJECT diverzoegerung[3][11]
+EXTERN OBJECT diverzoegerung[3][14]
 #ifdef XDIAL_C
 =
 {{
-  {11,0,0,38,8,OB_NONE,OB_NONE},
-  {OB_CENTERED|OB_UNDERLINED,1,1,36,1,TOB(TX_VERZOEGERUNG),OB_NONE},
-  {OB_CENTERED,1,3,36,1,TOB(TX_NIMM_STICH_NACH),OB_NONE},
-  {OB_CENTERED,1,4,36,1,OB_NONE,OB_NONE},
-  {OB_EXIT,2,6,3,1,OB_NONE,OB_NONE},
-  {OB_EXIT,6,6,3,1,OB_NONE,OB_NONE},
-  {OB_EXIT,10,6,3,1,OB_NONE,OB_NONE},
-  {OB_EXIT,14,6,10,1,TOB(TX_WEITER),OB_NONE},
-  {OB_EXIT,25,6,3,1,OB_NONE,OB_NONE},
-  {OB_EXIT,29,6,3,1,OB_NONE,OB_NONE},
-  {OB_EXIT,33,6,3,1,OB_NONE,OB_NONE}
+  {14,0,0,27,12,OB_NONE,OB_NONE},
+  {OB_CENTERED|OB_UNDERLINED,1,1,25,1,TOB(TX_VERZOEGERUNG),OB_NONE},
+  {OB_CENTERED,1,3,25,1,TOB(TX_NIMM_STICH_NACH),OB_NONE},
+  {OB_CENTERED,1,4,25,1,OB_NONE,OB_NONE},
+  {OB_EXIT,2,5,3,1,OB_NONE,OB_NONE},
+  {OB_EXIT,6,5,3,1,OB_NONE,OB_NONE},
+  {OB_EXIT,10,5,3,1,OB_NONE,OB_NONE},
+  {OB_EXIT,14,5,3,1,OB_NONE,OB_NONE},
+  {OB_EXIT,18,5,3,1,OB_NONE,OB_NONE},
+  {OB_EXIT,22,5,3,1,OB_NONE,OB_NONE},
+  {OB_CENTERED,1,7,25,1,TOB(TX_GEBEN),OB_NONE},
+  {OB_BUTTON,2,8,11,1,TOB(TX_SCHNELL),1},
+  {OB_BUTTON,14,8,11,1,TOB(TX_LANGSAM),-1},
+  {OB_EXIT,7,10,13,1,TOB(TX_WEITER),OB_NONE}
 }}
 #endif
 ;
@@ -599,7 +729,7 @@ EXTERN OBJECT dimenubutton[3][9]
   {OB_BUTTON,21,3,4,1,OB_NONE,1},
   {OB_BUTTON,26,3,4,1,OB_NONE,1},
   {OB_BUTTON,31,3,4,1,OB_NONE,-5},
-  {OB_EXIT,14,5,10,1,TOB(TX_WEITER),OB_NONE}
+  {OB_EXIT,12,5,14,1,TOB(TX_WEITER),OB_NONE}
 }}
 #endif
 ;
