@@ -29,12 +29,6 @@
 #include "xio.h"
 #include "text.h"
 
-int left(s)
-int s;
-{
-  return (s+1)%3;
-}
-
 VOID change_gc(sn,fg,gcp)
 int sn;
 unsigned long fg;
@@ -77,14 +71,14 @@ char *s;
   return !strcmp(h,"true");
 }
 
-VOID v_gtextc(sn,c,x,y,w,t)
-int sn,c,x,y,w;
+VOID v_gtextnc(sn,n,c,x,y,w,t)
+int sn,n,c,x,y,w;
 char *t;
 {
   int l;
 
   l=strlen(t);
-  x+=(w-XTextWidth(dfont[sn],t,l))/2;
+  if (n) x+=(w-XTextWidth(dfont[sn],t,l))/2;
   y+=(20*desk[sn].f/desk[sn].q-charh[sn])/2+dfont[sn]->ascent;
   if (c) {
     change_gcbg(sn,btpix[sn],gc);
@@ -96,6 +90,13 @@ char *t;
   }
 }
 
+VOID v_gtextc(sn,c,x,y,w,t)
+int sn,c,x,y,w;
+char *t;
+{
+  v_gtextnc(sn,1,c,x,y,w,t);
+}
+
 VOID v_gtext(sn,x,y,w,t)
 int sn,x,y,w;
 char *t;
@@ -188,8 +189,13 @@ int i,s;
       if (sn==s) y1[sn]=desk[sn].playy;
       putdesk(sn,x1[sn],y1[sn]);
     }
-    else {
-      if (stich==10) putdesk(sn,x1[sn],y1[sn]);
+    else if (stich==10) {
+      putdesk(sn,x1[sn],y1[sn]);
+    }
+    else if (spitzeang && cards[i]==(trumpf==4?BUBE:SIEBEN|trumpf<<3)) {
+      putback(sn,x1[sn],y1[sn]);
+      sptzmrk=1;
+      putamark(sn,spieler);
     }
     x2[sn]=desk[sn].stichx+vmh*desk[sn].cardw;
     y2[sn]=desk[sn].stichy;
@@ -304,24 +310,24 @@ int sn;
   desk[sn].com1y=desk[sn].com2y=desk[sn].y+3;
   if (ouveang && sn!=spieler) {
     if (spieler==left(sn)) {
-      desk[sn].com2y+=desk[sn].cardh+11;
+      desk[sn].com2y+=desk[sn].cardh+11*desk[sn].f/desk[sn].q;
     }
     else {
-      desk[sn].com1y+=desk[sn].cardh+11;
+      desk[sn].com1y+=desk[sn].cardh+11*desk[sn].f/desk[sn].q;
     }
   }
   desk[sn].cardx=(desk[sn].w-desk[sn].cardw)/9;
   desk[sn].playx=desk[sn].x+(desk[sn].w-desk[sn].cardw-9*desk[sn].cardx)/2;
   desk[sn].playy=desk[sn].y+desk[sn].h-desk[sn].cardh-2;
   desk[sn].skatx=desk[sn].x+(desk[sn].w-2*desk[sn].cardw)/2;
-  desk[sn].skaty=desk[sn].y+desk[sn].cardh+51;
+  desk[sn].skaty=desk[sn].y+desk[sn].cardh+51*desk[sn].f/desk[sn].q;
   desk[sn].stichx=desk[sn].x+(desk[sn].w-3*desk[sn].cardw)/2;
   desk[sn].stichy=desk[sn].skaty;
   desk[sn].cbox1x=desk[sn].com1x;
   desk[sn].cbox2x=desk[sn].com2x;
-  desk[sn].cboxy=desk[sn].com1y+desk[sn].cardh+11;
+  desk[sn].cboxy=desk[sn].com1y+desk[sn].cardh+11*desk[sn].f/desk[sn].q;
   desk[sn].pboxx=desk[sn].skatx;
-  desk[sn].pboxy=desk[sn].skaty+desk[sn].cardh+13;
+  desk[sn].pboxy=desk[sn].skaty+desk[sn].cardh+13*desk[sn].f/desk[sn].q;
 }
 
 VOID extractnam(sn,str)
@@ -349,14 +355,16 @@ VOID usage()
   fprintf(stderr,"\
 xskat [-display|-d display] [-geometry|-g geometry] [-font|-fn font]\n\
   [-iconic|-i] [-title|-T string] [-name prog] [-fg color] [-bg color]\n\
-  [-bt color] [-mb button] [-tdelay sec] [-help|-h] [-color] [-mono]\n\
-  [-color1 color] .. [-color20 color] [-large] [-small]\n\
+  [-bt color] [-mb button] [-tdelay sec] [-fastdeal] [-slowdeal] [-help|-h]\n\
+  [-color] [-mono] [-color1 color] .. [-color20 color] [-large] [-small]\n\
   [-up] [-down] [-alt] [-seq] [-list|-l filename] [-alist] [-nlist] [-tlist]\n\
   [-log filename] [-dolog] [-nolog] [-fmt] [-unfmt] [-game filename]\n\
   [-lang language] [-start player#] [-s1 -4..4] [-s2 -4..4] [-s3 -4..4]\n\
   [-ramsch] [-noramsch] [-ramschonly] [-sramsch] [-nosramsch]\n\
   [-kontra] [-nokontra] [-kontra18] [-bock] [-nobock] [-bockramsch]\n\
-  [-bockevents 1..255] [-resumebock] -[noresumebock]\n\
+  [-bockevents 1..255] [-resumebock] [-noresumebock]\n\
+  [-spitze] [-spitze2] [-nospitze] [-revolution] [-norevolution]\n\
+  [-klopfen] [-noklopfen] [-schenken] [-noschenken]\n\
   [-auto #ofgames] [-opt filename] [player@display...]\n\
 After starting the game a mouse click will bring up a menu.\n\
 ");
@@ -622,6 +630,50 @@ char *argv[];
       resumebock=0;
       resumebockset=1;
     }
+    else if (!strcmp(argv[1],"-spitze")) {
+      spitzezaehlt=1;
+      spitzezaehltset=1;
+    }
+    else if (!strcmp(argv[1],"-spitze2")) {
+      spitzezaehlt=2;
+      spitzezaehltset=1;
+    }
+    else if (!strcmp(argv[1],"-nospitze")) {
+      spitzezaehlt=0;
+      spitzezaehltset=1;
+    }
+    else if (!strcmp(argv[1],"-revolution")) {
+      revolution=1;
+      revolutionset=1;
+    }
+    else if (!strcmp(argv[1],"-norevolution")) {
+      revolution=0;
+      revolutionset=1;
+    }
+    else if (!strcmp(argv[1],"-klopfen")) {
+      klopfen=1;
+      klopfenset=1;
+    }
+    else if (!strcmp(argv[1],"-noklopfen")) {
+      klopfen=0;
+      klopfenset=1;
+    }
+    else if (!strcmp(argv[1],"-schenken")) {
+      schenken=1;
+      schenkenset=1;
+    }
+    else if (!strcmp(argv[1],"-noschenken")) {
+      schenken=0;
+      schenkenset=1;
+    }
+    else if (!strcmp(argv[1],"-fastdeal")) {
+      fastdeal=1;
+      fastdealset=1;
+    }
+    else if (!strcmp(argv[1],"-slowdeal")) {
+      fastdeal=0;
+      fastdealset=1;
+    }
     else if ((argv[1][0]!='-') && numsp<3) {
       disp_name[numsp++]=argv[1];
     }
@@ -685,7 +737,7 @@ char *argv[];
       else if (!strcmp(argv[1],"-start")) {
 	geber=atoi(argv[2]);
 	if (geber<1 || geber>3) geber=0;
-	else geber=(geber+1)%3;
+	else geber=left(geber);
       }
       else if (!strcmp(argv[1],"-auto")) {
 	numgames=atoi(argv[2]);
@@ -889,12 +941,34 @@ char *argv[];
 	(res=XGetDefault(dpy[sn],prog_name,"resumebock"))) {
       resumebock=istrue(res);
     }
+    if (!sn && !spitzezaehltset &&
+	(res=XGetDefault(dpy[sn],prog_name,"spitze"))) {
+      spitzezaehlt=atoi(res);
+      if (spitzezaehlt<0) spitzezaehlt=0;
+      else if (spitzezaehlt>2) spitzezaehlt=2;
+    }
+    if (!sn && !revolutionset &&
+	(res=XGetDefault(dpy[sn],prog_name,"revolution"))) {
+      revolution=istrue(res);
+    }
+    if (!sn && !klopfenset &&
+	(res=XGetDefault(dpy[sn],prog_name,"klopfen"))) {
+      klopfen=istrue(res);
+    }
+    if (!sn && !schenkenset &&
+	(res=XGetDefault(dpy[sn],prog_name,"schenken"))) {
+      schenken=istrue(res);
+    }
+    if (!sn && !fastdealset &&
+	(res=XGetDefault(dpy[sn],prog_name,"fastdeal"))) {
+      fastdeal=istrue(res);
+    }
     if (!sn && geber<0) {
       if ((res=XGetDefault(dpy[sn],prog_name,"start"))) {
 	geber=atoi(res);
       }
       if (geber<1 || geber>3) geber=0;
-      else geber=(geber+1)%3;
+      else geber=left(geber);
     }
     if (!sn) {
       for (x=0;x<2;x++) {
@@ -1159,7 +1233,7 @@ int sn,s;
 {
   int a,b,xp1,xp2,yp,xyarr[4];
 
-  if (sn==s || ouveang) return;
+  if (sn==s || ouveang || (spitzeang && !sptzmrk)) return;
   a=48*desk[sn].f/desk[sn].q-4;
   b=48*desk[sn].f/desk[sn].q-2;
   change_gc(sn,bpix[sn],gc);
@@ -1261,6 +1335,7 @@ int s,n;
 {
   int sn,sna[3],x1[3],y1[3],x2[3],y2[3];
 
+  sptzmrk=0;
   for (sn=0;sn<numsp;sn++) {
     sna[sn]=sn;
     if (s<0) {
@@ -1285,7 +1360,7 @@ int s,n;
     }
     else y2[sn]=desk[sn].playy;
   }
-  movecard(numsp,sna,x2,y2,x1,y1);
+  if (!fastdeal) movecard(numsp,sna,x2,y2,x1,y1);
   for (sn=0;sn<numsp;sn++) {
     putback(sn,x1[sn],y1[sn]);
     if (s==hoerer) putamark(sn,s);
@@ -1298,27 +1373,27 @@ int s,n;
       putback(sn,x1[sn]+desk[sn].cardw,y1[sn]);
     }
   }
-  waitt(300,2);
+  if (!fastdeal) waitt(300,2);
 }
 
 VOID initscr(sn)
 int sn;
 {
-  int i,x,y,w;
+  int i,x,y;
 
-  if (phase!=ANSAGEN) {
-    di_info(sn,-1);
-    if (predef && (!ouveang || sn==spieler)) {
-      x=(desk[sn].w-strlen(textarr[TX_VORDEFINIERTES_SPIEL])*charw[sn])/2;
-      y=desk[sn].y+2*charh[sn];
-      w=strlen(textarr[TX_VORDEFINIERTES_SPIEL])*charw[sn];
-      v_gtext(sn,x,y,w,textarr[TX_VORDEFINIERTES_SPIEL]);
-    }
-  }
+  if (phase==WEITER || phase==REVOLUTION) return;
   sort(sn);
   for (i=0;i<10;i++) {
     putcard(sn,cards[sn*10+i],desk[sn].playx+i*desk[sn].cardx,desk[sn].playy);
   }
+  if (phase!=ANSAGEN) {
+    di_info(sn,-1);
+    if (predef && (!ouveang || sn==spieler)) {
+      x=desk[sn].w/2;
+      y=desk[sn].y+2*charh[sn];
+      v_gtext(sn,x,y,0,textarr[TX_VORDEFINIERTES_SPIEL]);
+    }
+  }
   if (phase!=ANSAGEN && ouveang) {
     if (sn==spieler) {
       for (sn=0;sn<numsp;sn++) {
@@ -1335,17 +1410,129 @@ int sn;
       putback(sn,x,y);
     }
   }
+  else if (spitzeang && sn!=spieler) {
+    x=spieler==left(sn)?desk[sn].com1x:desk[sn].com2x;
+    y=spieler==left(sn)?desk[sn].com1y:desk[sn].com2y;
+    putcard(sn,trumpf==4?BUBE:SIEBEN|trumpf<<3,x,y);
+  }
 }
 
-VOID clr_desk()
+VOID nullendscr()
+{
+  int sn,s,i,d,x,y1,y2,sav[3];
+
+  clr_desk(0);
+  for (sn=0;sn<numsp;sn++) {
+    di_info(sn,-2);
+    for (s=0;s<3;s++) {
+      sav[0]=alternate[s];
+      sav[1]=sort1[s];
+      sav[2]=sort2[s];
+      alternate[s]=0;
+      sort1[s]=sort1[sn];
+      sort2[s]=sort2[sn];
+      sort(s);
+      alternate[s]=sav[0];
+      sort1[s]=sav[1];
+      sort2[s]=sav[2];
+    }
+    y1=desk[sn].com1y;
+    y2=desk[sn].skaty;
+    if (left(sn)!=spieler) swap(&y1,&y2);
+    d=sort1[sn]?stich-1:0;
+    for (i=0;i<11-stich;i++) {
+      x=desk[sn].playx+(stich-1+2*i)*desk[sn].cardx/2;
+      putcard(sn,cards[sn*10+i+d],x,desk[sn].playy);
+      s=left(sn);
+      putcard(sn,cards[s*10+i+d],x,y1);
+      s=left(s);
+      putcard(sn,cards[s*10+i+d],x,y2);
+    }
+    if (sn!=spieler)  {
+      x=desk[sn].skatx+desk[sn].cardx;
+      y1=(desk[sn].skaty+desk[sn].com1y+desk[sn].cardh-charh[sn])/2;
+      v_gtext(sn,x,y1,0,textarr[TX_NULL_DICHT]);
+    }
+  }
+  phase=WEITER;
+  di_weiter(1);
+}
+
+VOID revolutionsort(sp)
+int sp;
+{
+  int sn,s,i,x,y1,y2,sav[3];
+
+  for (s=0;s<3;s++) {
+    sav[0]=alternate[s];
+    sav[1]=sort1[s];
+    sav[2]=sort2[s];
+    alternate[s]=0;
+    sort1[s]=revolsort;
+    sort2[s]=1;
+    sort(s);
+    alternate[s]=sav[0];
+    sort1[s]=sav[1];
+    sort2[s]=sav[2];
+  }
+  for (sn=0;sn<numsp;sn++) {
+    if (sn==spieler) continue;
+    y1=desk[sn].com1y;
+    y2=desk[sn].skaty;
+    if (left(sn)!=spieler) swap(&y1,&y2);
+    for (i=0;i<10;i++) {
+      x=desk[sn].playx+i*desk[sn].cardx;
+      putcard(sn,cards[sn*10+i],x,desk[sn].playy);
+      s=left(sn);
+      if (sp || s!=spieler) putcard(sn,cards[s*10+i],x,y1);
+      s=left(s);
+      if (sp || s!=spieler) putcard(sn,cards[s*10+i],x,y2);
+    }
+  }
+}
+
+VOID revolutionscr()
+{
+  int sn,mi,x,y;
+
+  tauschcard=-1;
+  tauschdone=0;
+  revolsort=iscomp(spieler)?sort1[0]:sort1[spieler];
+  clr_desk(1);
+  for (sn=0;sn<numsp;sn++) {
+    if (sn==spieler) continue;
+    di_info(sn,-2);
+    x=desk[sn].skatx+desk[sn].cardx;
+    y=(desk[sn].skaty+desk[sn].com1y+desk[sn].cardh-charh[sn])/2;
+    v_gtext(sn,x,y,0,textarr[TX_KARTEN_AUSTAUSCHEN]);
+    mi=left(sn)==spieler?left(spieler):left(sn);
+    x=desk[sn].playx+10*desk[sn].f/desk[sn].q;
+    v_gtextnc(sn,0,0,x,desk[sn].com1y+desk[sn].cardh+1,
+	      0,textarr[spieler==ausspl?TX_VORHAND:
+			spieler==left(ausspl)?TX_MITTELHAND:
+			TX_HINTERHAND]);
+    v_gtextnc(sn,0,0,x,desk[sn].skaty-charh[sn]-1,
+	      0,textarr[mi==ausspl?TX_VORHAND:
+			mi==left(ausspl)?TX_MITTELHAND:
+			TX_HINTERHAND]);
+    put_fbox(sn,TX_FERTIG);
+  }
+  revolutionsort(1);
+  phase=REVOLUTION;
+}
+
+VOID clr_desk(nsp)
+int nsp;
 {
   int sn;
 
   for (sn=0;sn<numsp;sn++) {
-    backgr(sn,desk[sn].x,desk[sn].y,desk[sn].w,desk[sn].h);
-    di_info(sn,3);
+    if (!nsp || sn!=spieler) {
+      backgr(sn,desk[sn].x,desk[sn].y,desk[sn].w,desk[sn].h);
+      di_info(sn,3);
+    }
   }
-  if (ouveang) {
+  if (!nsp && ouveang) {
     for (sn=0;sn<numsp;sn++) {
       if (sn!=spieler) di_info(sn,-2);
     }
@@ -1439,11 +1626,9 @@ int s,c;
   }
 }
 
-VOID put_fbox(t)
-int t;
+VOID put_fbox(sn,t)
+int sn,t;
 {
-  int sn=spieler;
-
   draw_box(sn,desk[sn].pboxx+24*desk[sn].f/desk[sn].q,desk[sn].pboxy,
 	   80*desk[sn].f/desk[sn].q);
   v_gtextc(sn,1,desk[sn].pboxx+24*desk[sn].f/desk[sn].q,desk[sn].pboxy,
@@ -1451,18 +1636,16 @@ int t;
 	   textarr[t]);
 }
 
-VOID rem_fbox()
+VOID rem_fbox(sn)
+int sn;
 {
-  int sn=spieler;
-
   backgr(sn,desk[sn].pboxx+24*desk[sn].f/desk[sn].q,desk[sn].pboxy-5,
 	 80*desk[sn].f/desk[sn].q,28*desk[sn].f/desk[sn].q);
 }
 
-VOID inv_fbox()
+VOID inv_fbox(sn)
+int sn;
 {
-  int sn=spieler;
-
   change_gcxor(sn,btpix[sn]^fgpix[sn]^bgpix[sn]);
   XFillRectangle(dpy[sn],win[sn],gcxor[sn],
 		 desk[sn].pboxx+24*desk[sn].f/desk[sn].q+4,desk[sn].pboxy+1,
@@ -1473,25 +1656,19 @@ VOID inv_fbox()
   change_gcxor(sn,fgpix[sn]);
 }
 
-int card_at(sn,x,y)
-int sn,x,y;
+int card_at(sn,x,y,zw)
+int sn,x,y,zw;
 {
-  int c,i;
+  int c,s;
 
-  if (y>=desk[sn].playy) {
-    x-=desk[sn].playx+2;
-    if (x<0 || x%desk[sn].cardx>=60*desk[sn].f/desk[sn].q ||
-	x>=9*desk[sn].cardx+60*desk[sn].f/desk[sn].q) return 0;
+  zw=(zw && y>=desk[sn].skaty && y<desk[sn].skaty+desk[sn].cardh);
+  if (y>=desk[sn].playy || zw) {
+    x-=desk[sn].playx;
+    if (x<0) return 0;
     c=x/desk[sn].cardx;
     if (c>9) c=9;
-    if (cards[10*sn+c]>=0) return c+1;
-    for (i=0;i<2;i++) {
-      if (c--==0) return 0;
-      if (c*desk[sn].cardx<=x && x<c*desk[sn].cardx+60*desk[sn].f/desk[sn].q &&
-	  cards[10*sn+c]>=0) {
-	return c+1;
-      }
-    }
+    s=zw?left(sn)==spieler?left(spieler):left(sn):sn;
+    if (cards[10*s+c]>=0) return c+1+(zw?10:0);
   }
   return 0;
 }
@@ -1511,7 +1688,6 @@ int sn,x,y;
     if (b) maxrw[sn]=0;
     else {
       maxrw[sn]=215;
-      sagte18[sn]=1;
     }
     do_entsch();
     return 1;
@@ -1524,7 +1700,7 @@ int sn,x,y;
 {
   int c,sna[1],x1[1],y1[1],x2[1],y2[1];
 
-  c=card_at(sn,x,y);
+  c=card_at(sn,x,y,0);
   if (c) {
     c--;
     swap(&cards[10*sn+c],&cards[drkcd+30]);
@@ -1542,21 +1718,26 @@ int sn,x,y;
   if (x>=desk[sn].pboxx+31*desk[sn].f/desk[sn].q &&
       x<=desk[sn].pboxx+96*desk[sn].f/desk[sn].q &&
       y>=desk[sn].pboxy+1 &&
-      y<=desk[sn].pboxy+16*desk[sn].f/desk[sn].q) inv_fbox();
+      y<=desk[sn].pboxy+16*desk[sn].f/desk[sn].q) inv_fbox(spieler);
   else return 0;
   stdwait();
-  inv_fbox();
+  inv_fbox(spieler);
   if (trumpf==5 && (((cards[30]&7)==BUBE) || ((cards[31]&7)==BUBE))) {
     di_buben();
     return 1;
   }
-  rem_fbox();
+  rem_fbox(spieler);
   if (trumpf==5) {
     putback(sn,desk[sn].skatx,desk[sn].skaty);
     putback(sn,desk[sn].skatx+desk[sn].cardw,desk[sn].skaty);
-    vmh=(vmh+1)%3;
-    if (vmh) di_schieben();
-    else start_ramsch();
+    if (ramschspiele && klopfen) {
+      di_klopfen(spieler);
+    }
+    else {
+      vmh=left(vmh);
+      if (vmh) di_schieben();
+      else start_ramsch();
+    }
     return 1;
   }
   home_skat();
@@ -1573,12 +1754,90 @@ int sn,x,y;
   return 1;
 }
 
+int hndl_tauschen(sn,x,y)
+int sn,x,y;
+{
+  int c,c1,zw,mi,tf,d[2],sna[2],x1[2],y1[2],x2[2],y2[2];
+
+  tf=left(sn)==spieler?1:2;
+  mi=left(sn)==spieler?left(spieler):left(sn);
+  if (tauschdone&tf) return 0;
+  c=card_at(sn,x,y,1);
+  if (c) {
+    zw=c>10;
+    c=(c-1)%10;
+    c1=tauschcard!=-1?tauschcard%10:0;
+    x1[0]=desk[sn].playx+c*desk[sn].cardx;
+    y1[0]=zw?desk[sn].skaty:desk[sn].playy;
+    x2[0]=desk[sn].playx+c1*desk[sn].cardx;
+    y2[0]=zw?desk[sn].playy:desk[sn].skaty;
+    d[0]=(zw?4:-4)*desk[sn].f/desk[sn].q;
+    if (!iscomp(mi)) {
+      x1[1]=desk[mi].playx+c*desk[mi].cardx;
+      y1[1]=zw?desk[mi].playy:desk[mi].skaty;
+      x2[1]=desk[mi].playx+c1*desk[mi].cardx;
+      y2[1]=zw?desk[mi].skaty:desk[mi].playy;
+      d[1]=(zw?4:-4)*desk[mi].f/desk[mi].q;
+    }
+    if (tauschcard!=-1) {
+      if ((zw?mi:sn)==tauschcard/10) {
+	if (c==c1) return 1;
+	putdesk(sn,x2[0],y1[0]+d[0]);
+	putcard(sn,cards[tauschcard],x2[0],y1[0]);
+	if (!iscomp(mi)) {
+	  putdesk(mi,x2[1],y1[1]-d[1]);
+	  putcard(mi,cards[tauschcard],x2[1],y1[1]);
+	}
+      }
+      else {
+	swap(&cards[tauschcard],&cards[10*(zw?mi:sn)+c]);
+	sna[0]=sn;
+	if (!iscomp(mi)) sna[1]=mi;
+	movecard(iscomp(mi)?1:2,sna,x1,y1,x2,y2);
+	putdesk(sn,x2[0],y2[0]-d[0]);
+	if (!iscomp(mi)) putdesk(mi,x2[1],y2[1]+d[1]);
+	revolutionsort(0);
+	tauschcard=-1;
+	if (tauschdone) {
+	  put_fbox(mi,TX_FERTIG);
+	  tauschdone=0;
+	}
+	return 1;
+      }
+    }
+    tauschcard=10*(zw?mi:sn)+c;
+    putdesk(sn,x1[0],y1[0]);
+    putcard(sn,cards[tauschcard],x1[0],y1[0]+d[0]);
+    if (!iscomp(mi)) {
+      putdesk(mi,x1[1],y1[1]);
+      putcard(mi,cards[tauschcard],x1[1],y1[1]-d[1]);
+    }
+    return 1;
+  }
+  if (x>=desk[sn].pboxx+31*desk[sn].f/desk[sn].q &&
+      x<=desk[sn].pboxx+96*desk[sn].f/desk[sn].q &&
+      y>=desk[sn].pboxy+1 &&
+      y<=desk[sn].pboxy+16*desk[sn].f/desk[sn].q) inv_fbox(sn);
+  else return 0;
+  if (!iscomp(mi) && !tauschdone) inv_fbox(mi);
+  stdwait();
+  inv_fbox(sn);
+  if (!iscomp(mi) && !tauschdone) inv_fbox(mi);
+  rem_fbox(sn);
+  tauschdone|=tf;
+  if (iscomp(mi) || tauschdone==3) {
+    clr_desk(1);
+    spielphase();
+  }
+  return 1;
+}
+
 int hndl_spielen(sn,x,y)
 int sn,x,y;
 {
   int i,c;
 
-  c=card_at(sn,x,y);
+  c=card_at(sn,x,y,0);
   if (c) {
     di_delres(sn);
     c--;
@@ -1634,6 +1893,9 @@ int sn,x,y,mbt,pbt;
   else if (phase==NIMMSTICH) {
     if (nimmstich[sn][1]) ok=hndl_nimmstich(sn);
   }
+  else if (phase==REVOLUTION) {
+    if (sn!=spieler) ok=hndl_tauschen(sn,x,y);
+  }
   if (!ok) {
     if (!mbt || mbt==pbt) {
       di_options(sn);
@@ -1654,27 +1916,44 @@ int sn,x,y,mbt,pbt;
 
 VOID setcurs()
 {
-  int x,y,w,sn,snn,newsn=-1;
+  int x,y,w,sn,snn,newsn=-1,newsn2=-1;
   char clr[100];
-  static int actsn=-1,wsn=-1,wtime;
+  static int actsn=-1,actsn2=-1,wsn=-1,wtime;
 
   for (sn=0;sn<numsp;sn++) {
-    if (phase==REIZEN) {
+    switch (phase) {
+    case REIZEN:
       if ((saho && sn==sager) || (!saho && sn==hoerer)) newsn=sn;
-    }
-    else if (phase==HANDSPIEL || phase==DRUECKEN ||
-	     phase==ANSAGEN || phase==RESULT) {
+      break;
+    case HANDSPIEL:
+    case DRUECKEN:
+    case ANSAGEN:
+    case RESULT:
       if (phase==ANSAGEN && ktrply>=0) newsn=ktrply;
       else if (sn==spieler) newsn=sn;
       if (numsp==1) newsn=0;
-    }
-    else if (phase==SPIELEN) {
+      break;
+    case SPIELEN:
       if (sn==(ausspl+vmh)%3) newsn=sn;
-    }
-    else if (phase==NIMMSTICH) {
+      break;
+    case SCHENKEN:
+      newsn=schenkply;
+      break;
+    case NIMMSTICH:
       for (snn=0;snn<numsp;snn++) {
 	if (nimmstich[snn][1]) newsn=snn;
       }
+      break;
+    case NULLDICHT:
+    case WEITER:
+      newsn=dichtply;
+      break;
+    case REVOLUTION:
+      if (sn!=spieler) {
+	if (newsn==-1) newsn=sn;
+	else newsn2=sn;
+      }
+      break;
     }
   }
   if (newsn!=actsn) {
@@ -1683,35 +1962,42 @@ VOID setcurs()
     actsn=newsn;
     if (actsn>=0) {
       for (sn=0;sn<numsp;sn++) {
-	di_info(sn,actsn);
+	if (phase!=WEITER && phase!=REVOLUTION) di_info(sn,actsn);
       }
       XDefineCursor(dpy[actsn],win[actsn],cursor[actsn][0]);
     }
   }
+  if (newsn2!=actsn2) {
+    if (actsn2>=0 && actsn!=actsn2) {
+      XDefineCursor(dpy[actsn2],win[actsn2],cursor[actsn2][1]);
+    }
+    actsn2=newsn2;
+    if (actsn2>=0) {
+      XDefineCursor(dpy[actsn2],win[actsn2],cursor[actsn2][0]);
+    }
+  }
   if (numsp==1) return;
   if (actsn==-1) wtime=0;
   if (!wtime || wtime>=15*1000) {
     if (wsn!=-1 && (actsn!=wsn ||
 		    wtime==16*1000 ||
 		    (phase!=SPIELEN && phase!=NIMMSTICH))) {
-      x=(desk[wsn].w-30*charw[wsn])/2;
+      x=desk[wsn].w/2;
       y=desk[wsn].playy-2*charh[wsn];
       w=5*desk[wsn].cardw/XTextWidth(dfont[wsn]," ",1)+1;
       if (w>99) w=99;
       clr[w]=0;
       while (w) clr[--w]=' ';
-      w=30*charw[wsn];
-      v_gtext(wsn,x,y,w,clr);
+      v_gtext(wsn,x,y,0,clr);
       wsn=-1;
       if (wtime==16*1000) wtime-=1500;
       if (phase!=SPIELEN && phase!=NIMMSTICH) wtime=0;
     }
     if (wtime==15*1000 && (phase==SPIELEN || phase==NIMMSTICH)) {
       wsn=actsn;
-      x=(desk[wsn].w-strlen(textarr[TX_DU_BIST_DRAN])*charw[wsn])/2;
+      x=desk[wsn].w/2;
       y=desk[wsn].playy-2*charh[wsn];
-      w=strlen(textarr[TX_DU_BIST_DRAN])*charw[wsn];
-      v_gtext(wsn,x,y,w,textarr[TX_DU_BIST_DRAN]);
+      v_gtext(wsn,x,y,0,textarr[TX_DU_BIST_DRAN]);
     }
   }
   wtime+=50;
