@@ -32,7 +32,8 @@ EXTERN XFontStruct *dfont[3];
 EXTERN Pixmap bck[3];
 EXTERN Pixmap bwcards[3],colcards[3];
 EXTERN Cursor cursor[3][2];
-EXTERN int ktrply;
+EXTERN int ktrply,sptzmrk,dichtply,schenkply;
+EXTERN int revolsort,tauschcard,tauschdone;
 EXTERN struct
 {
   int large;
@@ -60,16 +61,16 @@ EXTERN XColor color[3][20]
   {0,0x0000,0xffff,0x0000},
   {0,0xffff,0xffff,0x0000},
   {0,0x0000,0x0000,0xffff},
-  {0,0xffff,0xd999,0xd999},
+  {0,0xffff,0xcccc,0xcccc},
   {0,0x0000,0x0000,0x0000},
   {0,0xcccc,0xcccc,0xcccc},
-  {0,0x7fff,0x7fff,0x7fff},
-  {0,0x7fff,0x0000,0x0000},
-  {0,0x0000,0x7fff,0x0000},
-  {0,0x7fff,0x7fff,0x0000},
-  {0,0x0000,0x0000,0x7fff},
-  {0,0x7fff,0x0000,0x7fff},
-  {0,0x0000,0x7fff,0x7fff},
+  {0,0x9999,0x9999,0x9999},
+  {0,0x9999,0x0000,0x0000},
+  {0,0x0000,0x9999,0x0000},
+  {0,0x9999,0x9999,0x0000},
+  {0,0x0000,0x0000,0x9999},
+  {0,0x9999,0x0000,0x9999},
+  {0,0x0000,0x9999,0x9999},
   {0,0x0000,0x0000,0x0000}
 }}
 #endif
